# needed for version numbers
import pygame, sys
import sqlalchemy as sqla
import Version
version=Version.version

### don't make changes below this line ################
spversion = "Childsplay version: %s" % version
plat = "Platform :%s" % sys.platform
pyversion = "Python version: %s" % sys.version.split('\n')[0]
pgversion = "Pygame version: %s" % pygame.version.ver
sdlversion = "SDL version: %s.%s.%s" % pygame.get_sdl_version()
sqlaversion = "Sqlalchemy version: %s" % sqla.__version__
# Needed for the option parser
optversion = '\n'.join(['-'*60,spversion,plat, pyversion, pgversion,\
                        sdlversion,sqlaversion,'-'*60])



# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
#
#           quiz.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# Super class for the quiz acts

#create logger, logger was setup in SPLogging
import logging

module_logger = logging.getLogger("childsplay.quiz_knowledge")

# standard modules you probably need
import os,sys

import pygame
from pygame.constants import *

import utils
from SPConstants import *
import SPSpriteUtils

# needed to import quizengine
#sys.path.insert(0, './lib')
from quizengine import Engine

class Activity:
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activty name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
  """

    def __init__(self,SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger =  logging.getLogger("childsplay.quiz.Activity")
        self.logger.info("Activity started")
        self.SPG = SPGoodies
        self.lang = self.SPG.get_localesetting()[0][:2]
        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.logo_pos = (600, 200 + self.blit_pos[1])
        self.theme = self.SPG.get_theme()
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        # The location of the activities Data dir
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','Quiz_Data')
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'quiz.rc'))
        self.rchash['theme'] = self.theme
        self.rchash['lang'] = self.lang
        self.logger.debug("found rc: %s" % self.rchash)
        p = os.path.join(self.CPdatadir,'good_%s.png' % self.lang)
        if not os.path.exists(p):
            p = os.path.join(self.CPdatadir,'thumbs.png')
        self.ThumbsUp = SPSpriteUtils.MySprite(utils.load_image(p))
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
        # Location of the alphabet sounds dir
        self.absdir = self.SPG.get_absdir_path()# alphabet sounds dir
        # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
        # it returns a reference to the special CPGroup
        self.actives = SPSpriteUtils.SPInit(self.screen,self.backgr)
          
        self.num_questions = int(self.rchash[self.theme]['questions'])
        self.AreWeDT = False
            
    def stop_sound(self):
        """Called by the core when the audio needs to stopped"""
        try:
            self.quizengine.stop_sound()
        except:
            pass
    
    def quiz_voice_changed(self,state):
        """Called by the core when the quiz voice audio setting is changed"""
        self.quizengine.unmute_exer_audio = state
    
    def observer(self, result):
        self.logger.debug("observer called with: %s" % result)
        if result > 0:
            self.levelupcount += 1
            self.score += int(result)
        else:
            self.levelupcount -= 1
        # here we pause between questions
        pygame.time.wait(1000)
        pygame.event.clear()
        
    def clear_screen(self):
        self.screen.blit(self.orgscreen,self.blit_pos)
        self.backgr.blit(self.orgscreen,self.blit_pos)
        pygame.display.update()

    def get_moviepath(self):
        movie = os.path.join(self.my_datadir,'help.avi')
        return movie

    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        self.actives.redraw()

    def get_helptitle(self):
        """Mandatory method"""
        return _("Quiz")
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "quiz"
    
    def get_help(self):
        """Mandatory methods"""
        text = [_("The aim of this activity:"),
                _("Answer the quiz questions."), 
        " ",
        _("At the top of the screen, you will see a general knowledge question."),
        _("Below it are a choice of possible answers.\nTouch the correct answer."), 
        " "]
        return text 
    
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return ''
    
    def get_extra_info(self):
        """Optional method to provide extra info to the user by adding it to the 
        help dialog"""
        return _("question id: %s") % self.quizengine.get_question_id()

    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Memory")
    
    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels") % number-of-levels"""
        return _("This activity has %s levels") % 6

    def start(self):
        """Mandatory method."""
        self._start_engine()
        self.quizengine.set_retry(int(self.rchash[self.theme]['retry']))
        self.SPG.tellcore_set_dice_minimal_level(1)
    
    def pre_level(self, level):
        """Mandatory method"""
        pass

    def dailytraining_pre_level(self, level):
        """Mandatory method"""
        pass

    def dailytraining_next_level(self, level, dbmapper):
        """Mandatory method.
        This should handle the DT logic"""
        self.AreWeDT = True
        self.num_questions = int(self.rchash[self.theme]['dt_questions'])
        self.next_level(level, dbmapper)
        return True

    def next_level(self,level,dbmapper):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        # TODO: what about the mapper ? we don't put total_questions nor total_wrongs in it
        self.level = level
        self.levelupcount = 1
        self.quizengine.clear()
        self.score = 0# this is score for dbase which are stored per level
        self.dbmapper = dbmapper
        return True
    
    def post_next_level(self):
        """Mandatory method.
        This is called once by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a separate thread like sound play."""
        self.dbmapper.insert('total_questions', self.num_questions)
        if self.level in (1, 2):
            self.maxpoints = 2
            self.quizengine.init_exercises(2, self.num_questions,self.level, self.AreWeDT)
        else:
            self.maxpoints = 4
            self.quizengine.init_exercises(4, self.num_questions, self.level, self.AreWeDT) 
        self.quizengine.next_question()
        
    def get_score(self,timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        m,s = timespend.split(':')
        seconds = int(m)*60 + int(s)
        
        N = float(self.num_questions)
        M = float(self.maxpoints)
        P = float(self.score)
        F = 7.0
        F1 = 3.0
        C = 0.2
        S = float(seconds)
        # N = Number of questions
        # M = maximum points per question
        # P = Users result in points
        # F = Fraction of total result related to points. (maximum result is 10)
        # F1 = Fraction of result related to time
        # C = multiplier
        # S = time spend in seconds
        points = max((F/100) * (P/(N*M/100)), 1.0)
        time = min(F1 / ((S/(N*2)) ** C), F1)
        self.logger.info("@scoredata@ %s level %s N %s P %s M %s S %s points %s time %s" %\
                          (self.get_name(),self.level, N, P,M, S, points, time))
        score = points + time
        result = points + time
        return result
    
    def stop_timer(self):
        """You *must* provide a method to stop timers if you use them.
        The SPC will call this just in case and catch the exception in case 
        there's no 'stop_timer' method""" 
        try:
            self.timer.stop()
        except:
            pass
        try:
            self.quizengine.stop_timers()
        except:
            pass

    def loop(self,events):
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""
        if self.quizengine.loop(events):
            #print "name", self.get_name(), self.level        
            if not self.AreWeDT:
                self.clear_screen()
                self.dbmapper.insert('total_wrongs', self.quizengine.get_totalwrongs())
#                self.ThumbsUp.display_sprite((229, 296))
#                pygame.time.wait(1500)
#                self.ThumbsUp.erase_sprite()
                if self.levelupcount >= int(self.rchash[self.theme]['autolevel_value']):
                    levelup = 1
                    self.SPG.tellcore_level_end(store_db=True,\
                                                level=min(6, self.level), \
                                                levelup=levelup)
                else:
                    self.SPG.tellcore_level_end(store_db=True, level=self.level)
            else:
                self.SPG.tellcore_level_end(store_db=False, level=self.level)
            
        return 
        



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2011 Stas Zykiewicz <stas.zytkiewicz@gmail.com>
#
#           SPConstants.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# Set the NoGtk constants to True if you don't want to use gtk stuff.
NoGtk = False
# Set this to which dbase we use.
# Currently these are supported: 'sqlite' and 'mysql'
WHICHDBASE = 'sqlite'

# Set the default milestone for the mail tickets script 
MILESTONE = 'BT2.3'

# DON'T CHANGE ANYTHING BELOW THIS LINE !!!!
# If you want to change the paths, change SPBasePaths.py

import sys

import SPBasePaths
import logging
import os
import time
import shutil
import pygame
import glob 
SPCmodule_logger = logging.getLogger("childsplay.SPConstants")

#module_logger.debug("Contents of os.environ: %s" % os.environ)
#module_logger.debug("Contents of sys.argv %s " % sys.argv)

from SPColors import *

# Import keymaps as they are now in a seperate module.
from SPKeyMaps import KeyMaps

# Which themes do we have?
SUPPORTEDTHEMES = ['default', 'childsplay','cognitionplay', 'seniorplay','braintrainer']
# Which keymaps do we have?
SUPPORTEDKEYMAPS = KeyMaps.supportedmaps
# The base of the schoolsplay installation, only used on win98?
BASEDIR = SPBasePaths.BASEDIR
# here are the activities data directories installed
ACTIVITYDATADIR = SPBasePaths.SHARELIBDATADIR
# gettext mo files location
LOCALEDIR = SPBasePaths.LOCALEDIR
# Localized soundfiles
ALPHABETDIR = SPBasePaths.ALPHABETDIR
WWWDIR = SPBasePaths.WWWDIR
GUITHEMESPATH = os.path.join(ACTIVITYDATADIR, 'SPData', 'gui','themes')
DEFAULTGUITHEMESPATH = os.path.join(GUITHEMESPATH,'default')

THEMESPATH = os.path.join(ACTIVITYDATADIR, 'SPData', 'themes')
DEFAULTTHEMESPATH = os.path.join(THEMESPATH,'default')

CORESOUNDSDIR = os.path.join(ACTIVITYDATADIR, 'SPData', 'base', 'sounds')

ACTDATADIR = os.path.join(ACTIVITYDATADIR, 'CPData')

# Users sp dir
HOME_DIR_NAME = '.schoolsplay.rc'
# Name of the SQLite users dbase
DBASE = 'sp_users.db'
# name of the SQLite content dbase
CONTENTDBASE = 'sp_content.db'


# language
LANG = 'en'
LOCALE_RTL = None

# Check if there's a schoolsplay dir in the home directory
if sys.platform == 'win32':
    PLATFORM = 'win32'
    try:
        HOMEDIR = os.path.join(os.environ['HOMEDRIVE'], os.environ['HOMEPATH'], HOME_DIR_NAME)
    except:
        # for win 98 ??
        HOMEDIR = os.path.join(BASEDIR, HOME_DIR_NAME)
        if not os.path.exists(HOMEDIR):
            os.makedirs(HOMEDIR) 
else:
    PLATFORM = 'All your platform are belong to us'
    if os.path.exists('/data/userdata'):# BTP production machine
        HOMEDIR = os.path.join('/data/userdata', HOME_DIR_NAME)
    else:
        try:
            HOMEDIR = os.path.join(os.environ['HOME'], HOME_DIR_NAME)
        except KeyError, info:
            print info
            HOMEDIR = os.path.abspath(sys.path[0])

# Create a schoolsplay directory and subdirectories.
START_POSTPULL = 1
if not os.path.exists(HOMEDIR):
    os.makedirs(HOMEDIR)
    ppl = ["%s\n" % f for f in glob.glob(os.path.join(BASEDIR, 'post_pull', '*.sh'))]
    ppl.sort()
    START_POSTPULL = len(ppl)
    f = open(os.path.join(HOMEDIR, 'post_pull'), 'w')
    f.writelines(ppl)
    f.close()

PSYCOPATH = os.path.join(HOMEDIR, 'schoolsplay_psyco.log')
DBASEPATH = os.path.join(HOMEDIR, DBASE)

TEMPDIR = os.path.join(HOMEDIR, 'tmp')
if os.path.exists(TEMPDIR):
    shutil.rmtree(TEMPDIR)
os.makedirs(TEMPDIR)
os.chmod(TEMPDIR,0777)
os.makedirs(os.path.join(TEMPDIR, 'birthday'))
os.chmod(os.path.join(TEMPDIR, 'birthday'),0777)


BASEPATH = os.getcwd()
# lockfile path used to set a lock to prevent multiple instances of childsplay
LOCKFILE = os.path.join(HOMEDIR, '.splock')
HOMEIMAGES = os.path.join(HOMEDIR, 'my_images')

# set font path and fontsize
TTFSIZE = 19# used for default ttf
TTF = os.path.join(ACTIVITYDATADIR, 'SPData', 'base', 'arial.ttf')
TTFBOLD = os.path.join(ACTIVITYDATADIR, 'SPData', 'base','DejaVuSansCondensed-Bold.ttf')
if os.path.exists(TTF):
    # default size for DejaVuSansCondensed
    TTFSIZE = 12

# These are for the pango fonts
P_TTFSIZE = 11
P_TTF = 'arial' #'bookman'

#[6, 94, 182, 270, 358, 446, 534, 622, 710]
CORE_BUTTONS_XCOORDS = range(6, 800, 91)

SPLC_TIMESTAMP = time.strftime("%y-%m-%d_%H-%M-%S", time.localtime())

IMAGE_EXT_PATTERN = ['*.jpg','*.JPG','*.jpeg','*.png', '.*tiff', '*.gif', '*.bmp']
IMAGE_EXT = ['.jpg','.JPG','.jpeg','.png', '.tiff', '.gif', '.bmp']

XML_FILES_WE_MUST_HAVE = ["SP_menu.xml"]

BUTTON_FEEDBACK_TIME = 200





# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
#
#           quizengine.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# Provides Super classes for the quiz activities.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("childsplay.quiz_text.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("childsplay.quizengine")

import os, sys
import random
import glob
import pygame
from pygame.constants import *
import types
import datetime
from SPConstants import *

from utils import load_image, char2surf, get_locale_local, MyError, load_music
from utils import current_time, read_rcfile, sleep
from SPSpriteUtils import SPSprite, SPInit
from SPWidgets import TransPrevNextButton, TransImgButton
from SPSoundServer import SoundServer
from SPORMs import type2table

class ParserError(Exception):
    pass


# special gamethemes for regional and personal quizzes which aren't part of the server_content table
REGIONAL_GAME_THEME = -2
PERSONAL_GAME_THEME = -1

class ContentFeeder:
    """Extracts content from the SQL tables."""
    def __init__(self, dbase, difficulty, tp, rchash, spgoodies):
        """Class that interacts with the xml parser to get content from the disk,
        check for completeness and return it to the caller.
        dbase - path to the dbase
        difficulty - which difficulty level we want questions from.
        tp - string  which type of questions we want.
        possible types are: history,math,picture,royal,sayings,text,melody
        """
        self.logger = logging.getLogger("childsplay.quizengine.ContentFeeder")
        self.logger.debug("start ContentFeeder with %s" % dbase)
        self.tp = tp
        self.SPG = spgoodies
        self.rchash = rchash
        self.difficulty = difficulty
        self.served_dict = {}
        self.served = []
        self.wrongexercises = []
        self._not_serve_all_content = False
        self.current_user_id = self.SPG.get_current_user_id()
        self.served_content_orm, self.served_content_session = self.SPG.get_served_content_orm()
        self.rowlist = []
        self.current_question_id = -1
        self.type2table = type2table
        self.year = None
        self.questions_order = {}
        # we make one query for the language code and use that for all queries
        lang = get_locale_local()[0].split('_')[0] # we filter on language, not country
        orm, session = self.SPG.get_orm('game_languages', 'content')
        query = session.query(orm).filter_by(lang = lang)
        self.lang = [l.ID for l in query.all()]
        session.flush()
        session.close()
        # and a query to get the template strings for additional content files like audio and images.
        orm, session = self.SPG.get_orm('game_filenames', 'content')
        query = session.query(orm)
        self.filename_template = {}
        for row in query.all():
            self.filename_template[row.tableName[5:]] = row.fileNameFormat
        session.flush()
        session.close()
        if tp == 'regional':
            self.whatarewe = 'regional'
        elif tp == 'personal':
            self.whatarewe = 'personal'
        else:
            self.whatarewe = 'regular'
    
    def _get_kind(self):
        return self.whatarewe
    
    def __del__(self):
        try:
            self.served_content_session.close()
        except:
            pass

    def serve_all_content(self, value):
        """When set to True we serve all content, False will serve only content which
        has audio"""
        self._not_serve_all_content = value
        self.rowlist = []# will force get_exercise to get new rows
        
    def _get_rows(self, tp, difficulty, year=None):
        self.logger.debug("_get_rows types:%s, difficulty:%s year:%s" % (tp, difficulty, year))
        lang = self.lang
        if tp == 'math':
            lang = [1] # math is the same for all languages.
        orm, session = self.SPG.get_orm('game_released_content', 'content')
        query = session.query(orm)
        if tp == 'picture':
            query = query.filter_by(module = 'quiz_pic')
            query = query.filter(orm.language.in_(lang))
            gt_list = [gt.game_theme for gt in query.all()]
        elif tp == 'regional':
            gt_list = [REGIONAL_GAME_THEME]
        elif tp == 'personal':
            gt_list= [PERSONAL_GAME_THEME]
        else:
            query = query.filter_by(module = 'quiz_'+tp)
            query = query.filter(orm.language.in_(lang))
            gt_list = [gt.game_theme for gt in query.all()]
        
        session.close()
        if year or self.year:
            if self.year:# needed when the questions list is empty and get_exercise queries for ids
                year = self.year
            orm, session = self.SPG.get_orm(type2table[tp], 'content')
            query = session.query(orm)
            query = query.filter(orm.language.in_(lang))
            query = query.filter(orm.year > year).filter(orm.year < year + 10)
            query = query.filter(orm.content_checked > 0)
            rows = [result for result in query.all()]
            all_ids = [row.CID for row in rows]
            if not all_ids or len(all_ids) < 10:
                self.logger.info("Too little questions found, serving all years")
                query = query.filter_by(difficulty = difficulty)
                rows = [result for result in query.all()]
                all_ids = [row.CID for row in rows]
            random.shuffle(rows)
            self.num_questions = len(rows)
            l = self.SPG.check_served_content(rows, \
                            int(self.rchash[self.rchash['theme']]['questions']), \
                            gt_list, all_ids)
            self.year = year
            #print "list of rows",len(rows),l
            session.close()
        else:
            # and now we get the contents from game_quiz* with game_theme(s),
            # difficulty and language.
            orm, session = self.SPG.get_orm(type2table[tp], 'content')
            query = session.query(orm)
            query = query.filter(orm.language.in_(lang))
            query = query.filter(orm.game_theme.in_(gt_list))
            query = query.filter(orm.difficulty == difficulty)
            query = query.filter(orm.content_checked > 0)
            if tp == 'melody' or self._not_serve_all_content:
                self.logger.debug("Serving only questions with audio")
                query = query.filter(orm.audiofiles == 5)
            else:
                self.logger.debug("Serving all questions")
            if tp == 'personal':
                query = query.filter(orm.user_id == self.current_user_id)
            rows_with_audio = [result for result in query.all()]
            self.logger.debug("found %s exercises with audio" % len(rows_with_audio))
            if len(rows_with_audio) < 10:
                orm, session = self.SPG.get_orm(type2table[tp], 'content')
                query = session.query(orm)
                query = query.filter(orm.language.in_(lang))
                query = query.filter(orm.game_theme.in_(gt_list))
                query = query.filter(orm.difficulty == difficulty)
                query = query.filter(orm.content_checked > 0)
                if tp == 'personal':
                    query = query.filter(orm.user_id == self.current_user_id)
                rows_without_audio = [result for result in query.all()]
                rows = rows_without_audio
            else:
                rows = rows_with_audio
            
            session.close()
            all_ids = [row.CID for row in rows]
            random.shuffle(rows)
            self.num_questions = len(rows)
            self.logger.debug("total rows: %s" % len(rows))
            l = self.SPG.check_served_content(rows, \
                            int(self.rchash[self.rchash['theme']]['questions']), \
                            gt_list, all_ids)
            #print ">>>>>>>>>>", len(l)
            
        return l
    
    def set_difficulty(self, difficulty, AreWeDT, year=None):
        self.logger.debug("set_difficulty: %s, year: %s" % (difficulty, year))
        self.AreWeDT = AreWeDT
        self.rowlist = []
        if self.difficulty != difficulty:
            self.logger.debug("difficulty change, remove wrong exercises")
            self.wrongexercises = []
        else:
            self.wrongexercises = self.have_wrong_exercise()
        if not self.rowlist or len(self.rowlist) < int(self.rchash[self.rchash['theme']]['questions']):
            self.rowlist = self._get_rows(self.tp, difficulty, year)
        self.WehaveWrong = False
        if self.wrongexercises:
            self.UseWrongExercises = True
        else:
            self.UseWrongExercises = False
        self.difficulty = difficulty
        self.logger.debug("UseWrongExercises set to %s" % self.UseWrongExercises)

    def get_exercise(self):
        """Returns a dict with the keys set to the question, correct answer
        and possible answers.
        """
        if not self.AreWeDT and self.UseWrongExercises and random.choice([1, 2, 3]) == 1:
            if self.wrongexercises:
                d, self.current_question_id = self.wrongexercises.pop()
                self.logger.debug("serving wrong question: %s" % self.current_question_id)
                self.logger.debug("question contents %s" % d)
                return d
            else:
                self.WehaveWrong = False  
        
        if not self.rowlist:
            self.logger.warning("no more questions, getting them again from the dbase")
            self.rowlist = self._get_rows(self.tp, self.difficulty)
            self.served = []
        row = self.rowlist.pop()
        ##### for debugging only ##########################
#        orm, session = self.SPG.get_orm('game_quizpic', 'content')
#        query = session.query(orm)
#        row = query.filter_by(CID = 2752).first()
#        session.close()
        ###################################################
        i = row.CID
        print row
        print row.CID
        bad = ('\t', '\r', '\n')
        rq = ''.join(c for c in row.question if c not in bad).strip()
        ra = ''.join(c for c in row.answer if c not in bad).strip()
        rw1 = ''.join(c for c in row.wrong1 if c not in bad).strip()
        rw2 = ''.join(c for c in row.wrong2 if c not in bad).strip()
        rw3 = ''.join(c for c in row.wrong3 if c not in bad).strip()
        if row.audiofiles == 5:
            templ = self.filename_template['quiztext']
            q = (rq, templ % (i,'question'))
            a = (ra, templ % (i,'answer'))
            p = [(rw1, templ % (i,'wrong 1')), \
                    (rw2, templ % (i,'wrong 2')),\
                    (rw3, templ % (i,'wrong 3'))] 
        else:   
            q = (rq, '')
            a = (ra, '')
            p = [(rw1, ''), (rw2, ''), (rw3, '')] 
        # data is an image or movie.
        if hasattr(row, 'file') and row.file:
            if self.tp == 'picture':
                dt = self.filename_template['quizpic'] % row.ID
            elif self.tp == 'regional':
                dt = self.filename_template['quizregional'] % row.ID
            elif self.tp == 'personal':
                dt = self.filename_template['quizpersonal'] % row.ID
        else:
            dt = ''
        d = {'question':q, 'answer':a, 'possibles':p, 'data':dt, 'speakerID':str(row.speakerID)}
        ############# for debugging only #################################
#        d = {'question':('Dit is een lange vraag van 69 characters lang dat is best lang toch ?', ''), \
#        'answer':('Antwoord a is ook erg lang wel 44 characters', ''), \
#        'possibles':[('Antwoord 1 is ook erg lang wel 44 characters', ''), \
#                     ('Antwoord 2 is ook erg lang wel 44 characters',''),  \
#                     ('Scheidsrechtersfluitje', '')],\
#                     'data':dt, 'speakerID':'0'}
        #########################################################################
        self.logger.debug("get_exercise serves question cid: %s" % i)
        self.served_dict[i] = (d, 0)# used for the wrong answered exercises

        self.current_question_id = i
        if not len(self.served) == 2:
            self.served.append(i)
        else:
            self.served[0] = self.served[1]
            self.served[1] = i
        # TODO: how to get the module name ?? XXX
        
        svc = self.served_content_orm(user_id=self.current_user_id, CID=i,\
                       game_theme_id=row.game_theme, \
                       module='', start_time=datetime.datetime.now(), \
                        count_served=1)
        self.served_content_session.add(svc)
        self.served_content_session.commit()
        return d

    def set_exercise_result(self, id, result):
        """Called by Engine._notify_obs"""
        self.logger.debug("set_exercise_result called with id:%s result:%s" % (id, result))
        if self.served_dict.has_key(id):
            self.served_dict[id] = (self.served_dict[id][0], result)
            if result:
                orm, session = self.SPG.get_orm('served_content', 'user')
                query = session.query(orm).filter_by(CID = id)
                query.update({'wrong': orm.wrong + 1})# this query holds one row
                session.commit()
                session.close()
        else:
            self.logger.warning("set_exercise_result: question %s is unkown" % id) 
    
    def have_wrong_exercise(self):
        self.wrongexercises = []
        for k, v in self.served_dict.items():
            if not v[1]:
                self.WehaveWrong = True
                self.wrongexercises.append((v[0], k))
        if self.wrongexercises:
            random.shuffle(self.wrongexercises)
            if len(self.wrongexercises) > 1:
                self.wrongexercises = self.wrongexercises[:len(self.wrongexercises)/2]
        return self.wrongexercises
    
    def get_num_questions(self):
        return self.num_questions
    def get_current_id(self):
        return self.current_question_id
    

class AudioPlayer:
    def __init__(self, audio, ad_audio, ss):
        self.logger = logging.getLogger("childsplay.quizengine.AudioPlayer")
        self.logger.debug("setup audio stream: %s" % audio)
        self.ss = ss
        if not os.path.splitext(audio['question'])[1]:
            self.logger.info("We don't have audio, set play to False")
            self.wehaveaudio = False
            return
        else:
            self.wehaveaudio = True
        keys = audio.keys()
        try:
            self.audiolist = [audio['silence_1000']]
            self.audiolist.append(audio['question'])
            keys.sort()
            for k in keys[:-3]:
                self.audiolist.append(audio['silence_1000'])
                ad_k = k.split('_')[1]
                self.audiolist.append(ad_audio[ad_k][0])
                self.audiolist.append(audio['silence_500'])
                self.audiolist.append(audio[k])
        except (AttributeError, IndexError), info:
            self.logger.warning("%s" % info)
            self.wehaveaudio = False
    
    def play(self):
        if not self.wehaveaudio:
            self._play_noaudio()# perhaps we do something in the future
            return
        self.logger.debug("Playing music stream")
        for item in self.audiolist:
            if self.ss.play(item):
                self.logger.error("The sound server has problems with: %s" % item)
                break
    
    def stop(self):
        try:
            self.ss.stop()
        except AttributeError:
            pass
        else:
            # needed to let the soundserver stop the current music
            pygame.time.wait(500)
            
    def _play_noaudio(self):
        pass

class Answer(SPSprite):
    def __init__(self, image, alt_image, result_img, sound, pos, cbf, state):
        SPSprite.__init__(self, image)
        self.logger = logging.getLogger("childsplay.quizengine.Answer")
        self.connect_callback(self._cbf, MOUSEBUTTONUP, state)
        self.moveto(pos, hide=True)
        self.alt_image = alt_image
        self.alt_image.blit(result_img, (0, 0))
        self.cbf = cbf
        self.sound = sound
        self.ImTouched = False
    
    def _display(self):
        self.logger.debug("_display called")
        self.image = self.alt_image
        self.display_sprite()
        #self.sound.play()

    def _cbf(self, sprite, event, data):
        self.logger.debug("_cbf called: %s" % data)
        if self.ImTouched:
            return 
        self.ImTouched = True
        self.image = self.alt_image
        self.display_sprite()
        #self.sound.play()
        self.cbf(self, data[0])
        return True
        
class Question(SPSprite):
    def __init__(self, image, pos):
        SPSprite.__init__(self, image)
        self.logger = logging.getLogger("childsplay.quizengine.Question")
        self.moveto(pos, hide=True)
        
class Engine:
    """Class that handles the sprites for the quizzes.
    All the quiz elements are sprites, the answers and prev/next are Button objects.
    The class uses the observer pattern to communicate with the caller
    """
    def __init__(self, quiz, spgoodies, observers, rchash=None):
        """Engine which controls the quiz sprites.
        quiz - String indicating which kind of quiz do we run.
               possible values: history,math,picture,royal,sayings,text,melody
        actives - SPGroup instance which is used by the caller in the eventloop
        observers - list with observer objects
        """
        self.logger = logging.getLogger("childsplay.quizengine.Engine")
        self.SPG = spgoodies
        self.theme = self.SPG.get_theme()
        self.lang = self.SPG.get_localesetting()[0][:2]
        self.parent_observers = observers
        if rchash:
            rchash['lang'] = self.lang

        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        
        self.actives = SPInit(self.screen,self.backgr)
        self.actives.set_onematch(True)
        
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','Quizengine_Data')
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
        self.my_rcfile = read_rcfile(os.path.join(self.my_datadir,'quizengine.rc'))
        self.quizpause = int(self.my_rcfile[self.theme]['pause'])
        self.quizpause_good = int(self.my_rcfile[self.theme]['pause_good'])
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.content_imgpath = os.path.join(self.SPG.get_libdir_path(),'CPData','DbaseAssets','Images')
        self.content_sndpath = os.path.join(self.SPG.get_libdir_path(),'CPData','DbaseAssets', 'Sounds')
        self.local_content_imgpath = os.path.join(HOMEDIR, self.theme, 'LocalAssets')
        
        # will we have local sound content ??
        #self.local_content_sndpath = os.path.join(self.SPG.get_libdir_path(),'CPData','DbaseAssets', 'Sounds')
        self.ttfpath = TTF
        
        dbpath = os.path.join(HOMEDIR, 'quizcontent.db')
        self.quiz = quiz
        try:
            self.CF = ContentFeeder(dbpath, 1, quiz, rchash, spgoodies)
        except Exception, info:
            self.logger.exception("Failed to start contentfeeder: %s" % info)
            raise MyError
        self.whatarewe = self.CF._get_kind()
        self.retry = 2# default values
        self.totalretry = 1
        self.answers_list = []
        self.currentscreen = None
        self.correct_answer = None
        self.unmute_exer_audio = self.SPG._unmute_quiz_voice
        self.CF.serve_all_content(self.unmute_exer_audio)
        p = os.path.join(self.CPdatadir,'good_%s.png' % self.lang)
        if not os.path.exists(p):
            p = os.path.join(self.CPdatadir,'thumbs.png')
        self.good_image = SPSprite(load_image(p))
        
        p = os.path.join(self.my_datadir,'tryagain_%s.png' % self.lang)
        if not os.path.exists(p):
            p = os.path.join(self.my_datadir,'tryagain_en.png')
        self.tryagain_image = SPSprite(load_image(p))
                
        iconA=load_image(os.path.join(self.my_datadir,'a.png'))
        iconB=load_image(os.path.join(self.my_datadir,'b.png'))
        iconC=load_image(os.path.join(self.my_datadir,'c.png'))
        iconD=load_image(os.path.join(self.my_datadir,'d.png'))
                
        y = self.blit_pos[1]
        self.question_pos = (5, 5 + y)
        self.answer2_pos = [((5, 200 + y),iconA, 'a'), ((5, 365 + y), iconB, 'b')]
        self.answer4_pos = [((5, 170 + y), iconA, 'a'), ((5, 255 + y), iconB, 'b'),\
                             ((5, 335 + y), iconC, 'c'), ((5, 415 + y), iconD, 'd')]
        self.logo_pos = (615, 200 + y)
        self.snd_pos = (700, 410 + y)
        self.prev_pos = (580, 410 + y)
        self.previous_screen_path = os.path.join(TEMPDIR, 'previous_screenshot.jpeg')
        self.data_display_center = (600, 200 + y)
        if self.quiz == 'picture':
            y = 0
        self.good_image.moveto((229, 280 + y))
        self.good_image.set_use_current_background(True)

        picture_button_width = 385
        
        self.question_surf_long = load_image(os.path.join(self.my_datadir,'vraagbox.png'))
        self.question_surf_short = load_image(os.path.join(self.my_datadir,'vraagbox_small.png'))
        
        self.answer2_surf_long = load_image(os.path.join(self.my_datadir,'box2.png'))
        self.answer2_surf_short = load_image(os.path.join(self.my_datadir,'box2_small.png'))
      
        self.answer2GR_surf_long = load_image(os.path.join(self.my_datadir,'box2_green.png'))
        self.answer2GR_surf_short = load_image(os.path.join(self.my_datadir,'box2_green_small.png'))

        self.answer2R_surf_long = load_image(os.path.join(self.my_datadir,'box2_red.png'))
        self.answer2R_surf_short = load_image(os.path.join(self.my_datadir,'box2_red_small.png'))
        
        self.answer4_surf_long = load_image(os.path.join(self.my_datadir,'box4.png'))
        self.answer4_surf_short = load_image(os.path.join(self.my_datadir,'box4_small.png'))
        
        self.answer4GR_surf_long = load_image(os.path.join(self.my_datadir,'box4_green.png'))
        self.answer4GR_surf_short = load_image(os.path.join(self.my_datadir,'box4_green_small.png'))
        
        self.answer4R_surf_long = load_image(os.path.join(self.my_datadir,'box4_red.png'))
        self.answer4R_surf_short = load_image(os.path.join(self.my_datadir,'box4_red_small.png'))
        
        self.good4_surf = load_image(os.path.join(self.my_datadir,'box4_good.png'))
        self.wrong4_surf = load_image(os.path.join(self.my_datadir,'box4_wrong.png'))
        
        # Setup some of the static objects
        prev = os.path.join(self.my_datadir,'core_prev_button.png')
        prev_ro = os.path.join(self.my_datadir,'core_prev_button_ro.png')
        next = os.path.join(self.my_datadir,'core_next_button.png')
        next_ro = os.path.join(self.my_datadir,'core_next_button_ro.png')
        self.prevnext_button = TransPrevNextButton(self.prev_pos, \
                                              self._cbf_prevnext_button, \
                                              prev, prev_ro, next, next_ro)
        
        self.snd_button = TransImgButton(os.path.join(self.my_datadir, 'speaker.png'), \
                                         os.path.join(self.my_datadir, 'speaker_ro.png'),\
                                         self.snd_pos)
        self.snd_button.connect_callback(self._cbf_snd_button, MOUSEBUTTONUP)
                            
        self.good_sound=os.path.join(self.CPdatadir,'good.ogg')
        self.wrong_sound=os.path.join(self.CPdatadir,'wrong.ogg')
        if self.quiz == 'picture':
            name = 'picture'
        else:
            name = self.quiz
        if self.lang == 'en':
            p = os.path.join(self.SPG.get_libdir_path(),'SPData', 'themes',\
                                        self.theme,'menuicons', \
                                    'quiz_%s.icon.png' % name)
        else:
            p = os.path.join(self.SPG.get_libdir_path(),'SPData', 'themes',\
                                        self.theme,'menuicons', self.lang, \
                                    'quiz_%s.icon.png' % name)
        self.logger.debug("loading logo %s" % p)
        try:
            self.logo_sprite = SPSprite(load_image(p))
        except Exception, info:
            self.logger.debug("No logo found for %s" % self.quiz)
            self.logo_sprite = None
        if self.quiz == 'picture':
            self.logo_sprite = None
        self.actives.add(self.prevnext_button.get_actives())
        self.current_question_id = -1
        # get a soundserver for the AudioPlayer object
        self.SS = SoundServer()
        self.scoredisplay = self.SPG.get_scoredisplay()
        self.previous_screen = None
    
    def stop_timers(self):
        self.logger.debug("stop_timers called")
        try:
            self.SS.stop_server(0)
        except:
            pass

    def _clear_screen(self):
        for s in self.answers_list:
            s.erase_sprite()
        self.screen.blit(self.orgscreen,self.blit_pos)
        self.backgr.blit(self.orgscreen,self.blit_pos)
        pygame.display.update()
        
        if self.logo_sprite:
            self.logo_sprite.display_sprite(self.logo_pos)
    
    def _cbf_snd_button(self, *args):
        self.SS.clear_queue()
        self.logger.debug('_snd_button_cbf called')
        self.audioplayer.play()
    
    def _cbf_prevnext_button(self, sprite, event, data):
        if data[0] == 'prev': 
            self._cbf_prev_button(data)
        else: 
            self._cbf_next_button(data)

    def _cbf_next_button(self, data):
        self.SS.clear_queue()
        self.logger.debug("_cbf_next_button: %s" % data)
        self._clear_screen()
        self.actives.add(self.answers_list)
        for s in self.answers_list:
            s.display_sprite()
        self.prevnext_button.toggle()
        if self.wehaveaudio:
            #self.snd_button.enable(True)
            self.snd_button.display_sprite()
        try:
            self.current_question_id = self.CF.served[1]
        except IndexError, info:
            self.logger.error(info)
            # this shouldn't happen
            self.current_question_id = -1
          
    def _cbf_prev_button(self, data):
        self.SS.clear_queue()
        self.logger.debug("_cbf_prev_button: %s" % data)
        self.actives.remove(self.answers_list)
        self._clear_screen()
        if self.logo_sprite:
            self.logo_sprite.erase_sprite()
        if os.path.exists(self.previous_screen_path):
            surf = load_image(self.previous_screen_path)
            pygame.display.update(self.screen.blit(surf, (0, self.blit_pos[1])))
        else:
            pygame.display.update(self.screen.blit(self.previous_screen, (0, self.blit_pos[1])))
        self.prevnext_button.toggle()
        if self.wehaveaudio:
            #self.snd_button.enable(False)
            self.snd_button.erase_sprite()
        try:
            self.current_question_id = self.CF.served[0]
        except IndexError, info:
            self.logger.error(info)
            self.current_question_id = -1
    
    def stop_sound(self):
        """Called by the parent if the sound needs to be stopped"""
        self.SS.clear_queue()
            
    def _cbf_answers(self, sprite, result):
        """Callback function for the answers.
        Here we determine the users result by calculating a result."""
        self.audioplayer.stop()
        self.SS.clear_queue()
        self.SS.play(sprite.sound)
        if self.erase_tryagain:
            self.tryagain_image.erase_sprite()
        # result == 0 -> good, result == 1 -> wrong
        self.logger.debug("_cbf_answers called with:%s,%s" % (sprite, result))
        if self.retry == None:# two answers
            if result == 0:
                self.good_image.display_sprite()
                sleep(self.quizpause_good)
                self.good_image.erase_sprite()
            else:
                self.maxresult = 0
                self.totalwrongs += 1
                sleep(self.quizpause)
            self._notify_obs(self.maxresult)
            return
        if self.retry and result == 0:# correct answer for 4 answers
            self.good_image.display_sprite()
            sleep(self.quizpause_good)
            self.good_image.erase_sprite()
            self._notify_obs(self.maxresult)
            return
        self.totalwrongs += 1
        self.maxresult -= result
        self.retry -= 1
        if self.retry == 0 and result:
            # whatever, your time has come to quit this pathetic attempt
            self.maxresult = 0
            self.correct_answer._display()
            sleep(self.quizpause)
            self._notify_obs(self.maxresult)
            return
        r = sprite.rect.move(100,0)
        self.tryagain_image.moveto((r[0], r[1]))
        self.tryagain_image.set_use_current_background(True)
        self.tryagain_image.display_sprite()
        self.erase_tryagain = True
        pygame.event.clear()
        
    def _notify_obs(self, result):
        """Notify the callers observers and restart the next question
        """
        self.logger.debug("_notify_obs called, result = %s" % result)
        # keep sprites for the 'previous screen'
        if self.CF.WehaveWrong and result > 1:
            self.scoredisplay.increase_score(result*5)
            self.parent_observers(result/2.0)
        elif result > 0:
            self.scoredisplay.increase_score(result*10)
            self.parent_observers(result)
        if result > -1:
            if result > 0:
                self.CF.set_exercise_result(self.current_question_id, 1)
            self.previous_screen = pygame.Surface((800, 500))
            surf = pygame.display.get_surface()
            self.previous_screen.blit(surf, (0, 0), (0, self.blit_pos[1], 800, 500))
            # we save it also on disk, this is needed for when we run under the generalknowledge quiz
            # we don't remove it, we let the core decide if it should be removed as
            # the core knows if we run under the general. 
            pygame.image.save(self.previous_screen, self.previous_screen_path)
            self.next_question()
        elif result == -1:
            self.logger.debug("_notify_obs level ended, doing nothing yet")
                
                
    def _blit_on_me(self, surf, ls, x=10, y=5):
        for s in ls:
            surf.blit(s, (x, y))
            y += s.get_height()
        return surf
        
    def set_retry(self, retry):
        """Used by the quiz caller to set the number of times the user 
        may have a retry when an answer is wrong.
        This is only used by 4 answers of course and only values 0-3 are allowed.
        A value of zero means no retry.
        retry - integer."""
        self.logger.debug("retry set to %s" % retry)
        self.retry = retry
        self.totalretry = retry
    
    def _do_data(self, data):
        """Here we decide what to do with the data.
        This is called if we are a picture quiz"""
        self.logger.debug("_do_data called with: %s" % data)
        if self.logo_sprite:
            self.logo_sprite.erase_sprite()
        if os.path.splitext(data)[1] in ('.png', '.jpg', 'gif'):
            if self.whatarewe == 'regular':
                s = load_image(os.path.join(self.content_imgpath, data))
            elif self.whatarewe == 'regional':
                s = load_image(os.path.join(self.local_content_imgpath, 'Regional',data))
            elif self.whatarewe == 'personal':
                s = load_image(os.path.join(self.local_content_imgpath, 'Personal', \
                                            str(self.SPG.get_current_user_id()),data))
            else:
                s = load_image(os.path.join(self.content_imgpath, data))
            pygame.draw.rect(s, ROYALBLUE, s.get_rect(), 4)
            self.picture_sprite = SPSprite(s)
            self.picture_sprite.rect.center = self.data_display_center
            #self.picture_sprite.display_sprite()
            self.answers_list.append(self.picture_sprite)
            return 'short' # indicating we used parts of the screen and left less space for question etc 
        elif os.path.splitext(data)[1] in ('.mpg', '.mpeg'):
            pass
            #TODO: placeholder for movie stuff

    def get_question_id(self):
        """return the id of the question that's currently displayed."""
        return self.current_question_id

    def clear(self):
        """Remove all actives sprites from the actives group and erase them"""
        if not self.answers_list:
            return
        self.logger.debug("Clearing sprites")
        self.actives.remove(self.answers_list)
        self._clear_screen()
        # check to see if the soundserver still running.
        self.SS.start_server()

    def init_exercises(self,answers, maxtimes, difficulty=1, AreWeDT=False, year=None):
        """Setup some defaults for the contentfeeder.
        answers - Integer indicating how many answers we should display.
        maxtimes - the number of questions we show, 0 means until we ran out of questions
        difficulty - level of difficulty.
        """
        self.totalwrongs = 0
        if not difficulty:
            difficulty = 1
        self.logger.debug("init_exercises: answers %s, maxtimes %s, difficulty %s, DT %s year %s" % \
                          (answers, maxtimes, difficulty, AreWeDT, year))
        self.SPG.dm.reset()
        # The first time were called by the caller quiz, then _next_question is called.
        # After that next_question is called in the _notify_obs method which is in turn called by the answer object.
        self.AreWeDT = AreWeDT
        
        self.CF.set_difficulty(difficulty, AreWeDT, year)
        if self.CF.get_num_questions() < 10:
            self.logger.error("To little questions found in %s for locale '%s', found %s questions should be at least 10" % (self.quiz, self.lang, self.CF.get_num_questions()))
            self.SPG.tellcore_info_dialog( _("To little questions found in %s for locale '%s', found %s questions should be at least 10"% (self.quiz, self.lang, self.CF.get_num_questions())))
            raise ParserError,"To little questions found in %s for locale '%s', found %s questions should be at least 10" % (self.quiz, self.lang, self.CF.get_num_questions())
        
        self.answers = answers
        if not maxtimes:
            self.maxtimes = self.CF.get_num_questions()
        else:
            self.maxtimes = maxtimes
        self.numOfQuestions = 0
        self.audiohash = {'0':{'a':[], 'b':[], 'c':[], 'd':[]}}
        for i in os.listdir(os.path.join(self.SPG.get_libdir_path(), \
                                'CPData', 'Quizcontent', 'speakers')):
            p = os.path.join(self.SPG.get_libdir_path(), 'CPData', 'Quizcontent',\
                              'speakers', i, '*.ogg')
            ad_audiohash = {'a':[], 'b':[], 'c':[], 'd':[]}
            for f in glob.glob(p):
                p, fp = os.path.split(f)
                ad_audiohash[fp[0]].append(f)
            self.audiohash[i] = ad_audiohash
        #self.prevnext_button.enable(False)
        
    def get_totalwrongs(self):
        return self.totalwrongs    
                
    def next_question(self):
        self.logger.debug("next_question called")
#        print self.numOfQuestions,self.maxtimes
        if self.numOfQuestions < self.maxtimes:
            self.numOfQuestions += 1
            self.logger.debug("serve question #%s" % self.numOfQuestions)
            if not self._next_question():
                self.maxresult = -1# notify observer level end
                self._notify_obs(self.maxresult)
        else:
            self.maxresult = -1# notify observer level end
            self._notify_obs(self.maxresult)
    
    def _next_question(self):
        # This uses pygame font rendering iso pango like the rest of this app.
        # The bigger pango fonts are not so nice.
        # There's a chance that these fonts are not rendered properly in non-western
        # languages.
        if self.quiz == 'math':
            fsize = 40
        else:
            fsize = 32
        fnt = self.ttfpath
        self.erase_tryagain = False
        audiohash = {'silence_1000': os.path.join(self.SPG.get_libdir_path(), \
                        'CPData', 'Quizcontent', 'silence_1000.ogg'), 
                        'silence_500': os.path.join(self.SPG.get_libdir_path(), \
                        'CPData', 'Quizcontent', 'silence_500.ogg')}
        
        self._clear_screen()
        self.actives.remove(self.answers_list)
        self.answers_list = []
        self.retry = self.totalretry
        exer = self.CF.get_exercise()
        if not exer:
            return
        if exer['data']:
            result = self._do_data(exer['data'])
        else:
            result = None
        if result == 'short':
            self.question_surf = self.question_surf_short
            self.answer2_surf = self.answer2_surf_short
            self.answer2R_surf = self.answer2R_surf_short
            self.answer2GR_surf = self.answer2GR_surf_short
            self.answer4_surf = self.answer4_surf_short
            self.answer4R_surf = self.answer4R_surf_short
            self.answer4GR_surf = self.answer4GR_surf_short
            split = 23
            ans_split = 28
            fsize = 30
            if self.answers == 2:
                ans_split = 23
        else:
            self.question_surf = self.question_surf_long
            self.answer2_surf = self.answer2_surf_long
            self.answer2R_surf = self.answer2R_surf_long
            self.answer2GR_surf = self.answer2GR_surf_long
            self.answer4_surf = self.answer4_surf_long
            self.answer4R_surf = self.answer4R_surf_long
            self.answer4GR_surf = self.answer4GR_surf_long
            split = 47
            ans_split = 36
                
        qs = self.question_surf.convert_alpha()
        text_blit_x = 50
        txt, audio = exer['question']
        self.logger.debug("We have audio :%s" % (audio != ''))
        if not self.unmute_exer_audio:
            self.logger.debug("quizvoice is disabled by user")
            if not self.quiz == 'melody':
                audio = ''
        # we will alter the audiohash in case we are melodyquiz AND voice is disabled
        # We will play the question but no answers
        audiohash['question'] = os.path.join(self.content_sndpath, audio)
        speakerID = exer['speakerID']
        if self.audiohash.has_key(speakerID):
            ad_audiohash = self.audiohash[speakerID]
        if audio:
            self.wehaveaudio = True
            self.logger.debug("We have audio, enable soundbutton")
            self.actives.add(self.snd_button)
        else:
            self.logger.debug("We don't have audio, disable soundbutton")
            self.wehaveaudio = False
            self.actives.remove(self.snd_button)
        ts = char2surf(txt, fsize,ttf=fnt, bold=False, fcol=ANTIQUEWHITE, split=split)
        qs = self._blit_on_me(qs, ts)
        qs = Question(qs, self.question_pos)
        qs.display_sprite()
        self.answers_list.append(qs)
        if self.answers == 2:
            self.maxresult = 2# used as a score result
            self.retry = None
            # first we get the correct answer
            txt_c, audio_c = exer['answer']
            # then we pick a random wrong answer
            txt_w, audio_w = random.choice(exer['possibles'])
            
            # blit the correct answer
            s_c = self.answer2_surf.convert_alpha()
            ts = char2surf(txt_c, fsize,ttf=fnt, bold=False,fcol=ANTIQUEWHITE, split=ans_split)
            s_c = self._blit_on_me(s_c, ts, x=50, y=24)
            # and the green one
            s_c_g = self.answer2GR_surf.convert_alpha()
            s_c_g = self._blit_on_me(s_c_g, ts, x=50, y=24)
            
            # blit the wrong anwser
            ts = char2surf(txt_w, fsize, ttf=fnt, bold=False, fcol=ANTIQUEWHITE, split=ans_split)  
            s_w = self.answer2_surf.convert_alpha()
            s_w = self._blit_on_me(s_w, ts, x=50, y=24)
            # and the red one
            s_w_r = self.answer2R_surf.convert_alpha()
            s_w_r = self._blit_on_me(s_w_r, ts, x=50, y=24)
            
            # determine the position (random).
            random.shuffle(self.answer2_pos)
            s_c = self._blit_on_me(s_c, [self.answer2_pos[0][1]])
            s_c_g = self._blit_on_me(s_c_g, [self.answer2_pos[0][1]])
            b = Answer(s_c, s_c_g, self.good4_surf, self.good_sound, \
                       self.answer2_pos[0][0], self._cbf_answers, 0)
            k = 'answer_%s' % self.answer2_pos[0][2]
            audiohash[k] = os.path.join(self.content_sndpath, audio_c)
            self.answers_list.append(b)
            self.correct_answer = b
            
            s_w = self._blit_on_me(s_w, [self.answer2_pos[1][1]])
            s_w_r = self._blit_on_me(s_w_r, [self.answer2_pos[1][1]])
            b = Answer(s_w, s_w_r, self.wrong4_surf, self.wrong_sound, \
                       self.answer2_pos[1][0], self._cbf_answers, 1)
            k = 'answer_%s' % self.answer2_pos[1][2]
            audiohash[k] = os.path.join(self.content_sndpath, audio_w)
            self.answers_list.append(b)

        else: # answer is 4
            self.maxresult = 4# used as a score result
            # first we get the correct answer
            txt_c, audio = exer['answer']
           
            random.shuffle(self.answer4_pos)
            # blit the correct one
            s_c = self.answer4_surf.convert_alpha()
            ts = char2surf(txt_c, fsize-8, ttf=fnt, bold=False,fcol=ANTIQUEWHITE, split=ans_split)
            s_c = self._blit_on_me(s_c, ts,  x=50)
            # and the green one            
            s_c_g = self.answer4GR_surf.convert_alpha()
            s_c_g = self._blit_on_me(s_c_g, ts, x=50)
            s_c = self._blit_on_me(s_c, [self.answer4_pos[0][1]])
            s_c_g = self._blit_on_me(s_c_g, [self.answer4_pos[0][1]])
            b = Answer(s_c, s_c_g, self.good4_surf, self.good_sound, \
                       self.answer4_pos[0][0], self._cbf_answers, 0)
            k = 'answer_%s' % self.answer4_pos[0][2]
            audiohash[k] = os.path.join(self.content_sndpath, audio)
            self.answers_list.append(b)
            self.correct_answer = b
            
            # here we setup the three incorrect answers
            random.shuffle(exer['possibles'])
            for i in range(1, 4):
                txt, audio = exer['possibles'][i-1]
                # placeholder: build audio stream
                # blit the wrong one
                ts = char2surf(txt, fsize-8, ttf=fnt, bold=False, fcol=ANTIQUEWHITE, split=ans_split)  
                s = self.answer4_surf.convert_alpha()
                s = self._blit_on_me(s, ts, x=50)
                s = self._blit_on_me(s, [self.answer4_pos[i][1]])
                k = 'answer_%s' % self.answer4_pos[i][2]
                audiohash[k] = os.path.join(self.content_sndpath, audio) 
                # and the red one
                sr = self.answer4R_surf.convert_alpha()
                sr = self._blit_on_me(sr, ts, x=50)
                sr = self._blit_on_me(sr, [self.answer4_pos[i][1]])
                b = Answer(s, sr, self.wrong4_surf, self.wrong_sound, \
                           self.answer4_pos[i][0], self._cbf_answers, 1)
                self.answers_list.append(b)
        
        # show the stuff       
        for s in self.answers_list:
            s.display_sprite()
        #self.prevnext_button.display_sprite()
        self.actives.add(self.answers_list)
        if self.numOfQuestions > 1 or os.path.exists(self.previous_screen_path):
            self.prevnext_button.enable(True)
            self.prevnext_button.display_sprite()
        else:
            self.prevnext_button.enable(False)
            self.prevnext_button.erase_sprite()
        if self.quiz == 'melody' and not self.unmute_exer_audio:
            # voice disabled but we are melody so  we want to play the question
            self.logger.debug("Altered audiosequence as we are melody and voice is disabled")
            self.audioplayer  = load_music(audiohash['question'])
        else:    
            self.audioplayer = AudioPlayer(audiohash, ad_audiohash, self.SS)
        self.audioplayer.play()
        if self.wehaveaudio:
            self.snd_button.display_sprite()
        self.current_question_id = self.CF.get_current_id()
        return True

    def loop(self, events):
        for event in events:
            if event.type in (MOUSEBUTTONUP, MOUSEBUTTONDOWN, MOUSEMOTION):
                result = self.actives.update(event)
        return self.maxresult == -1
    
if __name__ == '__main__':
    import SPLogging
    SPLogging.set_level('debug')
    SPLogging.start()
    
    # Test ContentFeeder
    db = os.path.join(HOMEDIR, 'quizcontent.db') 
    tp = 'math_nl'
    dif = '1'
    cf = ContentFeeder(db, dif, tp)
    cf.get_exercise()
    
    
    
    



# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
#
#           quiz_history.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("childsplay.quiz_history.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("childsplay.quiz_history")

# standard modules you probably need
import os,sys
# needed to import quiz
#sys.path.insert(0, './lib')

import pygame
from pygame.constants import *

import utils
from SPConstants import *
import SPSpriteUtils
import SPWidgets
import quiz
import SPWidgets
from SPVirtkeyboard import KeyBoard

# containers that can be used globally to store stuff
class Img:
    pass
class Snd:
    pass

class Activity(quiz.Activity):
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activty name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
  """

    def __init__(self,SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        quiz.Activity.__init__(self, SPGoodies)
        self.logger =  logging.getLogger("childsplay.quiz_history.Activity")
        self.logger.info("Activity started")
                
        # The location of the activities Data dir (override the super class attributes)
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','Quiz_historyData')
        self.background_keyboard = utils.load_image(os.path.join(self.my_datadir,'background_keyboard.png'))
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'quiz_history.rc'))
        self.rchash['theme'] = self.theme
        self.rchash['lang'] = self.lang
        
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        if self.blit_pos[1] == 0:
            y = 10
        else:
            y = 110
        # setup icons fro prelevel
        pos = [(20, 20+y), (240, 20+y), (460, 20+y), \
               (20, 240+y), (240, 240+y), (460, 240+y)]
        self.prelevel_buttons = []
        img = utils.load_image(os.path.join(self.my_datadir, '30s.png'))
        rop =  os.path.join(self.my_datadir, '30s_ro.png')
        if os.path.exists(rop):
            img_ro = utils.load_image(rop)
        else:
            img_ro = None
        but = SPWidgets.TransImgButton(img, img_ro, pos[0], name='30')
        but.connect_callback(self._prelevel_cbf, MOUSEBUTTONUP, 30)
        self.prelevel_buttons.append(but)
        
        img = utils.load_image(os.path.join(self.my_datadir, '40s.png'))
        rop =  os.path.join(self.my_datadir, '40s_ro.png')
        if os.path.exists(rop):
            img_ro = utils.load_image(rop)
        else:
            img_ro = None
        but = SPWidgets.TransImgButton(img, img_ro, pos[1], name='40')
        but.connect_callback(self._prelevel_cbf, MOUSEBUTTONUP, 40)
        self.prelevel_buttons.append(but)
        
        img = utils.load_image(os.path.join(self.my_datadir, '50s.png'))
        rop = os.path.join(self.my_datadir, '50s_ro.png')
        if os.path.exists(rop):
            img_ro = utils.load_image(rop)
        else:
            img_ro = None
        but = SPWidgets.TransImgButton(img, img_ro, pos[2], name='50')
        but.connect_callback(self._prelevel_cbf, MOUSEBUTTONUP, 50)
        self.prelevel_buttons.append(but)
        
        img = utils.load_image(os.path.join(self.my_datadir, '60s.png'))
        rop =  os.path.join(self.my_datadir, '60s_ro.png')
        if os.path.exists(rop):
            img_ro = utils.load_image(rop)
        else:
            img_ro = None
        but = SPWidgets.TransImgButton(img, img_ro, pos[3], name='60')
        but.connect_callback(self._prelevel_cbf, MOUSEBUTTONUP, 60)
        self.prelevel_buttons.append(but)
        
        img = utils.load_image(os.path.join(self.my_datadir, '70s.png'))
        rop = os.path.join(self.my_datadir, '70s_ro.png')
        if os.path.exists(rop):
            img_ro = utils.load_image(rop)
        else:
            img_ro = None
        but = SPWidgets.TransImgButton(img, img_ro, pos[4], name='70')
        but.connect_callback(self._prelevel_cbf, MOUSEBUTTONUP, 70)
        self.prelevel_buttons.append(but)
        
        img = utils.load_image(os.path.join(self.my_datadir, self.lang,'age.png'))
        rop =  os.path.join(self.my_datadir, self.lang, 'age_ro.png')
        if os.path.exists(rop):
            img_ro = utils.load_image(rop)
        else:
            img_ro = None
        but = SPWidgets.TransImgButton(img, img_ro, pos[5], name='age')
        but.connect_callback(self._prelevel_cbf, MOUSEBUTTONUP, 'age')
        self.prelevel_buttons.append(but)
        
    def _start_engine(self):
        try:
            self.quizengine = quiz.Engine('history', self.SPG,\
                                                   self.observer, self.rchash)
        except Exception, info:
            self.logger.exception("error starting quiz engine: %s" % info)
            self.SPG.tellcore_info_dialog(str(info))
    
    def start(self):
        """Mandatory method."""
        self._start_engine()
        self.quizengine.set_retry(int(self.rchash[self.theme]['retry']))
        self.SPG.tellcore_set_dice_minimal_level(5)
        self.num_questions = int(self.rchash[self.theme]['questions'])

    def pre_level(self, level):
        """Mandatory method"""
        self.logger.debug("pre_level called with: %s" % level)
        self.SPG.tellcore_disable_level_indicator()
        self.pre_level_flag = True 
        self.stop_prelevel_loop = False
        for b in self.prelevel_buttons:
            b.display_sprite()
        self.actives.add(self.prelevel_buttons)    
        return True
        
    def _kb_cbf(self, parent, data):
        year = int(data)
        if year < 100:
            year += 1900
        if not year or year < 1912:
            year = 1912
        if year > 1957:
            year = 1957
        self.year = year + 18
        self.logger.debug("setting year to %s" % self.year)
        parent.hide()
        del parent
        self.stop_prelevel_loop = True
        
    def _prelevel_cbf(self, sprite, event, data):
        self.logger.debug("_prelevel_cbf called with %s" % data)
        kind = data[0]
        for b in self.prelevel_buttons:
            b.erase_sprite()
        self.actives.remove(self.prelevel_buttons) 
        if kind != 'age':
            self.year = kind + 1900
            self.logger.debug("setting year to %s" % self.year)
            self.stop_prelevel_loop = True
            return True
        else:
            try:
                year = int(self.SPG.get_current_user_dbrow().birthdate.year)
            except (AttributeError, ValueError):
                year = 1930 
            else:
                if year > 1900:
                    if year < 1957:
                        self.year = year + 18
                    else:
                        self.year = 1975
                    self.logger.debug("setting year to %s" % self.year)
                    self.stop_prelevel_loop = True
                    return True

            pygame.display.update(self.screen.blit(self.background_keyboard,(115,140)))
            lbl = SPWidgets.Label(_("Please enter your year of birth."),\
                                (125, 150),fsize=24, fgcol=(255,255,255), transparent=True)
            lbl.display_sprite()
            kb = KeyBoard(self._kb_cbf, self.actives, keyboard_mode='numbers',\
                           position=(262, 300), \
                      wordcompletion=False, \
                      display_position=(262, 250),maxlen=4, display_length=0)
            kb.show()
        
    def post_next_level(self):
        """Mandatory method.
        This is called once by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a separate thread like sound play."""
        if self.level == 1:
            self.maxpoints = 2
            self.quizengine.init_exercises(2, self.num_questions, 1, self.AreWeDT,year=self.year)
        else:
            self.maxpoints = 4
            self.quizengine.init_exercises(4, self.num_questions, 1,self.AreWeDT, year=self.year) 
        self.quizengine.next_question()

    def get_helptitle(self):
        """Mandatory method"""
        return _("Quiz history")
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "quiz_history"
    
    def get_help(self):
        """Mandatory methods"""
        text = [_("The aim of this activity:"),
                 _("Answer the quiz questions."), 
        " ",
        _("At the top of the screen, you will see a question from general knowledge question from the selected period."),
        _("Below it are a choice of possible answers.\nTouch the correct answer."), 
        " "]
        return text 
    
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return ''
        
    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     language, Alphabet, Fun, Miscellaneous
        return _("Miscellaneous")
    
    
    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the following format:
        _("This level has %s levels") % number-of-levels"""
        return _("This activity has %s levels") % 2

    def loop(self,events):
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""
        # we override the loop in our parent so we must make the proper calls ourselfs
        if not self.pre_level_flag:
            if self.quizengine.loop(events):
                #print "name", self.get_name(), self.level
                self.clear_screen()
                self.ThumbsUp.display_sprite((180, 100))
                pygame.time.wait(1500)
                self.ThumbsUp.erase_sprite()
                if self.AreWeDT:
                    self.SPG.tellcore_level_end(level=self.level)
                if self.levelupcount >= int(self.rchash[self.theme]['autolevel_value']):
                    levelup = 1
                    self.SPG.tellcore_level_end(store_db=True,\
                                                level=min(6, self.level), \
                                                levelup=levelup)
                else:
                    self.SPG.tellcore_level_end(store_db=True, level=self.level)
        else:
            for event in events:
                if event.type in (MOUSEBUTTONUP, MOUSEMOTION):
                    self.actives.update(event)
                    if self.stop_prelevel_loop:
                        self.logger.debug("loop, got a result and stop this loop")
                        self.SPG.tellcore_pre_level_end()
                        self.pre_level_flag = False
                        self.SPG.tellcore_enable_level_indicator()
                        break                        
        return 



# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
#
#           memory_sp.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("schoolsplay.memory_sp.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("schoolsplay.memory_sp")

# standard modules you probably need
import os, sys, glob, random, textwrap

import pygame
from pygame.constants import *

import utils
from SPConstants import *
import SPSpriteUtils

import SPWidgets

# containers that can be used globally to store stuff
class Img:
    pass


class Snd:
    pass


class Global:
    selected_cards = ''
    displayedscore = 0


class Card(SPSpriteUtils.SPSprite):
    # This is a global reference to hold a selected object.
    # Used like this Selected is the same in all card classes
    # So when we set Selected in class0 it's also set in class1, class2 etc
    # see the callback method on how we use Selected
    Selected = None

    def __init__(self, closedcard, opencard, name, scoreobs, sound):
        """This class has it's own callback function and it will connect itself"""
        # The image passed to SPSprite will become self.image
        SPSpriteUtils.SPSprite.__init__(self, opencard)# embed it in this class
        self.name = name
        self.closedimage = closedcard.copy()
        self.openimage = opencard# image of a open card
        # connect to the MOUSEBUTTONDOWN event, no need for passing data
        self.connect_callback(self.callback, MOUSEBUTTONDOWN)
        self.scoreobs = scoreobs
        self.Sound = sound

    def close_card(self):
        self.image = self.closedimage
        self.display_sprite()

    def set_number(self, i, fs):
        s = utils.char2surf(str(i), fsize=fs, fcol=WHITE)
        sr = s.get_rect()
        sr.center = self.rect.center
        self.closedimage.blit(s, sr.topleft)


    def callback(self, sprite, event, *args):
        """ we will show the openimage when the user hits us"""
        # we check the 'global' attribute first 
        if not Card.Selected: # nothing selected yet
            # nothing selected, setting selected to this sprite
            self.Sound.button_hover.play()
            Card.Selected = self
            # keep track on how many times this card is opened
            # this is also stored in the dbase
            Global.selected_cards[self] += 1
            self.image = self.openimage
            self.display_sprite()
            return
        # remember Card.Selected holds a sprite of the other card
        elif Card.Selected is self:
            # user hits same card
            self.Sound.wrong.play()
            Card.Selected.image = Card.Selected.closedimage
            Card.Selected.display_sprite()
            Card.Selected = None
            return
            # we are the second card
        self.image = self.openimage
        self.display_sprite()
        Global.selected_cards[self] += 1
        if Card.Selected.name is sprite.name: # cards are the same  
            # "hooray" we found the second one
            self.Sound.good.play()
            pygame.time.wait(1000)
            self.remove_sprite()
            Card.Selected.remove_sprite()
            Card.Selected = None
            self.scoreobs(10)
        else:
            # bummer, wrong card
            self.Sound.wrong.play()
            pygame.time.wait(1000)
            # reset this card
            self.image = self.closedimage
            self.display_sprite()
            # reset previous selected card
            Card.Selected.image = Card.Selected.closedimage
            Card.Selected.display_sprite()
            Card.Selected = None
            self.scoreobs(-10)


class Activity:
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activty name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
  """

    def __init__(self, SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger = logging.getLogger("schoolsplay.memory_sp.Activity")
        self.logger.info("Activity started")
        self.SPG = SPGoodies
        self.scoredisplay = self.SPG.get_scoredisplay()
        self.theme = self.SPG.get_theme()
        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        # The location of the activities Data dir
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(), 'CPData', 'Memory_spData')
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'memory_sp.rc'))
        self.rchash['theme'] = self.theme
        self.logger.debug("found rc: %s" % self.rchash)
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(), 'CPData')
        # Location of the alphabet sounds dir
        self.absdir = self.SPG.get_absdir_path()# alphabet sounds dir
        # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
        # it returns a reference to the special CPGroup
        self.actives = SPSpriteUtils.SPInit(self.screen, self.backgr)
        files = ('dealcard1.wav', 'good.ogg', 'wrong.ogg', 'button_hover.wav')
        self.Sound = Snd()
        for file in files:
            setattr(self.Sound, file[:-4], utils.load_sound(os.path.join(self.CPdatadir, file)))
        
        self.beamer_set = 'off'
    
    def clear_screen(self):
        self.screen.blit(self.orgscreen, self.blit_pos)
        self.backgr.blit(self.orgscreen, self.blit_pos)
        pygame.display.update()

    def get_moviepath(self):
        movie = os.path.join(self.my_datadir, 'help.avi')
        return movie

    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        self.actives.redraw()

    def get_helptitle(self):
        """Mandatory method"""
        return _("Memory")

    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "memory_sp"

    def get_help(self):
        """Mandatory methods"""
        text = [_("The aim of this activity:"),
                _("Classic memory game where you have to find pairs of cards."),
                " ",
                _(
                    "Look closely at the pictures, when you have them memorized touch 'START'.\nThe pictures turn over, you must match the pair of cards."),
                " "]
        return text

    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return _("Accuracy is more important than speed.")

    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Memory")

    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels") % number-of-levels"""
        return _("This activity has %s levels") % 6

    def dailytraining_pre_level(self, level):
        """Mandatory method"""
        # TODO: it's hardcoded, for 2.2 we must have it in the rcfile
        self.tile = 'tileset_1'

    def dailytraining_next_level(self, level, dbmapper):
        """Mandatory method.
        This should handle the DT logic"""
        self.AreWeDT = True
        self.SPG.tellcore_disable_level_indicator()
        self.next_level(level, dbmapper)
        return True

    def stop(self):
        """Mandatory method, will be called by the core when this activities is destroyed."""
        pass

    def start(self):
        """Mandatory method."""
        self.tile = 'tileset_1'
        self.displaydscore = 0
        self.SPG.tellcore_set_dice_minimal_level(1)
        self.pre_level_flag = False
        self.start_loop_flag = False
        self.levelrestartcounter = 0
        self.level = 1
        # here we setup the card layout for each level (6 levels), list[0] is level 1 ect
        # (4,2) means 4 collomns of 2 rows 
        self.level_layout = [(3, 2), (4, 2), (4, 3), (4, 4), (5, 4), (6, 4)]
        self.AreWeDT = False
        self.filepaths = []

    def pre_level(self, level):
        """Mandatory method
        Return True if you want the core to enter the eventloop after this call."""
        # This sets up the "choose tile" screen and places the buttons in the actives
        # group. Because we return True MainCore starts calling our event loop.
        # In the eventloop we check the return values from the actives if one returns
        # a value we ask the core to stop this level, we handle the return value,
        # set some attributes and break the eventloop.
        # The core then calls the next_level method and eventloop.
        self.logger.debug("pre_level called with %s" % level)
        self.SPG.tellcore_disable_score_button()
        self.level = level
        self.pre_level_flag = True
        if self.blit_pos[1] == 0:
            y = 10
        else:
            y = 110
        x = 100

        files = glob.glob(os.path.join(self.my_datadir, 'tileset_*.png'))
        files.sort()
        lbl = SPWidgets.Label(_("Please, choose a set of cards."),\
            (30, y), fsize=24, transparent=True)
        lbl.display_sprite()
        y += lbl.get_sprite_height()
        for line in textwrap.wrap(_("The difficulty of the set increases from left to right."), 40):
            lbl = SPWidgets.Label(line, (30, y), fsize=24, transparent=True)
            lbl.display_sprite()
            y += lbl.get_sprite_height()
        y += 50
        tmp_img = utils.load_image(files[0])
        count = 1
        total_width = ((len(files) / 2) * (tmp_img.get_width())) + (((len(files) / 2) - 1) * 20)
        first_x = (self.screen.get_width() / 2) - (total_width / 2)
        for tilepath in files:
            x = first_x + (count - 1) * tmp_img.get_width() + (count - 1) * 20
            img = utils.load_image(tilepath)
            data = os.path.splitext(os.path.split(tilepath)[1])[0]
            if not data[-2:] == 'ro':
                img_ro = utils.load_image(os.path.join(self.my_datadir, '%s_ro.png' % data))
                b = SPWidgets.SimpleTransImgButton(img, img_ro, (x, y), data=data)
                b.display_sprite()
                self.actives.add(b)
                count += 1
        return True

    def start_exercise(self):
        imagelist = []
        # load all the images we need
        try:
            imgdir = os.path.join(self.my_datadir, self.tile, self.theme)
            if not os.path.exists(imgdir):
                imgdir = os.path.join(self.my_datadir, self.tile, 'childsplay')
                # load cardbacks and empty cards were we will blit our images on
            if self.rchash.has_key(self.theme):
                cardfront = self.rchash[self.theme]['cardfront']
                cardback = self.rchash[self.theme]['cardback']
            else:
                cardfront = self.rchash['childsplay']['cardfront']
                cardback = self.rchash['childsplay']['cardback']
            if self.beamer_set == 'on':
                cardback = "%s_beamer.png" % cardback.split('.png')[0]
            cardback = utils.load_image(os.path.join(imgdir, cardback))
            # the rest of the images are turn into sprites in the start method
            emptycard = utils.load_image(os.path.join(imgdir, cardfront))
            items = ('cardback.png', 'cardfront.png',\
                     'CP_cardback.png', 'CP_cardfront.png',\
                     'cardback_beamer.png')
            if len(self.filepaths) < self.num_of_cards / 2:
                self.filepaths = [f for f in glob.glob(os.path.join(imgdir, '*'))\
                                  if os.path.basename(f) not in items]
                random.shuffle(self.filepaths)
            files = self.filepaths[0:self.num_of_cards / 2]
            self.filepaths = self.filepaths[self.num_of_cards / 2:]
            self.logger.debug("Starting to load %s files" % len(files))
            for imgfile in files:
                # load and put the images in a list together with their file name
                # which we use as a id for the sprite later on
                file = os.path.basename(imgfile)
                img = utils.load_image(imgfile)
                imagelist.append((img, file))
        except (StandardError, utils.MyError), info:
            self.logger.exception("Can't load images for sprites: %s" % info)
            raise utils.MyError(str(info))# MyError will make the core end this game
        else:
            self.logger.info("Loaded %s images" % len(imagelist))
        imagelist = imagelist * 2 # we must have doubles of course        
        #random.shuffle(imagefiles)# needs to be shuffled
        if emptycard.get_alpha():
            cimg = emptycard.convert_alpha()
        else:
            cimg = emptycard.convert()
            # Unpack our image list and blit them on scaled cards

        # must we scale? we only test y if y fits x will always fit as the diff 
        # between x and y is never greater then 2
        # max y screen is 490, standard card size is 150x150

        cards = []
        cx, cy = self.level_layout[self.level - 1]# levels start with 1
        scalesize = 490 // cy - 10
        if scalesize < 120 or cx > 6:# first scale cards
            if cx > 6: scalesize = 96
            scaledcardback = utils.aspect_scale(cardback, (scalesize, scalesize))
            for img, file in imagelist:
                if emptycard.get_alpha():
                    cimg = emptycard.convert_alpha()
                else:
                    cimg = emptycard.convert()
                cimg = utils.aspect_scale(cimg, (scalesize, scalesize))
                cardcenter = cimg.get_rect().center
                s = utils.aspect_scale(img, (scalesize - 15, scalesize - 15))
                # center the image on the card
                sc = s.get_rect()
                sc.center = cardcenter
                cimg.blit(s, sc)
                cardsprite = Card(scaledcardback, cimg, file, self.scoreobs, self.Sound)
                cards.append(cardsprite)
        else:
            for img, file in imagelist:
                if emptycard.get_alpha():
                    cimg = emptycard.convert_alpha()
                else:
                    cimg = emptycard.convert()
                cardcenter = cimg.get_rect().center
                s = utils.aspect_scale(img, (120, 120))
                # center the image on the card
                sc = s.get_rect()
                sc.center = cardcenter
                cimg.blit(s, sc)
                cardsprite = Card(cardback, cimg, file, self.scoreobs, self.Sound)
                cards.append(cardsprite)
        random.shuffle(cards)
        if scalesize > 136:
            step = 160
        else:
            step = scalesize + 8
            # Your top blit position, this depends on the menubar position
        if self.blit_pos[1] == 0:
            y = 10
        else:
            y = 110
        start_x = (800 - (cx * step)) // 2 - 20
        start_y = (500 - (cy * step)) // 2 + y - 15

        # used to record how many times the same card is shown.
        # we store it into a class namespace to make it globally available.
        Global.selected_cards = {}
        i = 1
        size = 67 - (self.level * 3)
        for yy in range(cy):
            for x in range(cx):
                try:
                    sprite = cards.pop()
                except IndexError:
                    #in case of a level with lesser cards on the last line, like level 6
                    break
                Global.selected_cards[sprite] = 0
                if self.beamer_set == 'on':
                    sprite.set_number(i, size)
                    i += 1
                sprite.display_sprite((start_x + (x * step), start_y + (yy * step)))
                self.actives.add(sprite)
        pygame.display.update()
        # start display cards waiting loop 
        self.cards_list = self.actives.get_sprites()
        self.actives.empty()
        #        self.startbutton = SPWidgets.SimpleButtonDynamic(_("Start"), (690, y+430), fsize=24, data='start')
        self.startbutton = SPWidgets.SimpleButtonDynamic(_("Start"), (0, 0), fsize=24, data='start')
        x = 790 - self.startbutton.get_sprite_width()
        self.startbutton.moveto((x, y + 430))
        self.startbutton.set_use_current_background(True)
        self.startbutton.display_sprite()
        self.actives.add(self.startbutton)

        self.start_loop_flag = True

    def next_level(self, level, dbmapper):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        self.logger.debug("next_level called with %s" % level)
        if level > 6: return False # We only have 6 levels
        self.actives.empty()# remove sprites from pre_level
        if level == self.level:
            self.levelrestartcounter += 1
        self.level = level
        self.SPG.tellcore_enable_score_button()
        # db_mapper is a Python class object that provides easy access to the 
        # dbase.
        self.db_mapper = dbmapper
        Card.Selected = None
        # store number of cards into the db table 'cards' col
        a, b = self.level_layout[level - 1]
        if level == 6:
            self.num_of_cards = a * b - 2
        else:
            self.num_of_cards = a * b
        self.db_mapper.insert('cards', self.num_of_cards)
        # reset the screen to clear any crap from former levels
        self.clear_screen()

        self.start_exercise()

        return True

    def post_next_level(self):
        """Mandatory method.
        This is called once by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a seperate thread like sound play."""
        self.SPG.tellcore_enable_score_button()

    def scoreobs(self, i):
        """Called by the card object withmemory_sp.py a value to display as the score."""
        self.displaydscore += i
        if self.displaydscore <= 0:
            self.displaydscore = 0
        self.scoredisplay.set_score(self.displaydscore * 5)

    def get_score(self, timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        # T = Total cards
        # K = How many times the cards are selected. (a perfect game K == T)
        # F = Fraction of total result related to points. (maximum result is 10)
        # F1 = Fraction of result related to time
        # C = multiplier
        # S = time spend in seconds
        try:
            T = float(self.num_of_cards)
            K = float(self.knowncards)
        except AttributeError:
            return None
        m, s = timespend.split(':')
        S = float(int(m) * 60 + int(s))
        F = 8.0
        F1 = 2.0
        C = 0.6
        C1 = 0.7
        points = F / max(K / T, 1.0) ** C1
        time = F1 / max(S / T, 1.0) ** C
        self.logger.info("@scoredata@ %s level %s T %s K %s S %s points %s time %s" %\
                         (self.get_name(), self.level, T, K, S, points, time))
        score = points + time
        return score

    def stop_timer(self):
        """You *must* provide a method to stop timers if you use them.
        The SPC will call this just in case and catch the exception in case 
        there's no 'stop_timer' method"""
        try:
            self.timer.stop()
        except:
            pass

    def loop(self, events):
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""
        for event in events:
            if event.type in (MOUSEBUTTONUP, MOUSEBUTTONDOWN, MOUSEMOTION):
                result = self.actives.update(event)
                # were in prelevel loop
                if self.pre_level_flag:
                    if result:
                        self.logger.debug("loop, got a result and stop this loop")
                        self.SPG.tellcore_pre_level_end()
                        self.pre_level_flag = False
                        self.tile = result[0][1][0]
                        break
                    # were in "start" loop
                if self.start_loop_flag:
                    if result:
                        self.startbutton.erase_sprite()
                        self.actives.empty()
                        self.actives.add(self.cards_list)
                        self.Sound.dealcard1.play()
                        for c in self.actives:
                            c.close_card()
                        self.start_loop_flag = False

                # were in a normal game event loop
                if not self.actives.sprites():
                    # no more sprites left so the level is done.
                    # count the times the cards are turned.
                    # a lower value would be better, ideally a number equal to
                    # the number of cards would be a perfect score.
                    num = 0
                    for n in Global.selected_cards.values():
                        num += n
                        # store into dbase
                    self.db_mapper.insert('knowncards', num)
                    # set an attribute so that 'get_score' can calculate the score
                    self.knowncards = num
                    # we call the SPGoodies observer to notify the core the level
                    # is ended and we want to store the collected data
                    if self.levelrestartcounter >= int(self.rchash[self.theme]['autolevel_value']):
                        levelup = 1
                        self.levelrestartcounter = 0
                    else:
                        levelup = 0
                    if self.AreWeDT:
                        self.SPG.tellcore_level_end(level=self.level)
                    else:
                        self.SPG.tellcore_level_end(store_db=True,\
                            level=min(6, self.level),\
                            levelup=levelup)
        return 
                



# -*- coding: utf-8 -*-
 
# Copyright (c) 2010 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           SPVideoPlayer.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
import os
import subprocess

class _Cmd(object):
    # class that provides shell command strings
    start = "/usr/bin/vlc --one-instance-when-started-from-file --intf 'skins2' --skins2-last=%s %s"

class Player:
    """class that wraps the system call to the vlc player and provides
    some abstraction. It also restores the volume when vlc quits."""
    def __init__(self,skinpath):
        self.skinpath = os.path.abspath(skinpath)
        self._cmd = _Cmd()
        # Default is used in case of an error when calling amixer
        self.volume_level = 75

    def start(self, moviepath):
        """Start player.
        Returns (True,'') when successful and (False,text) on any error.
        This call blocks the main thread until the player is stopped."""
        self._get_volume()
        result = self._execute(self._cmd.start % (self.skinpath, os.path.abspath(moviepath))) 
        self._restore_volume()
        return result
        
    def stop(self):
        pass
    
    # internally used
    def _get_volume(self):
        try:
            cmd=subprocess.Popen("amixer get Master",shell=True, \
                                 stdout=subprocess.PIPE, \
                                 stderr=subprocess.PIPE)
            output = cmd.communicate()[0]
        except Exception, info:
            module_logger.warning("program 'amixer' not found, unable to get volume levels")
        else:
            for line in output.split('\n'):
                if "%]" in line:
                    self.volume_level = int(line.split("%]")[0].split("[")[1])
                    
    def _restore_volume(self):
        subprocess.Popen("amixer set Master %s" % self.volume_level + "%",shell=True)
        
    def _execute(self,cmd):
        try:
            #result = subprocess.call(cmd)
            result = os.system(cmd)
        except OSError,info:
            return (False,  cmd+" "+str(info))
        if result:
            return (False, str(result))
        return (True,'')


if __name__ == '__main__':
    pl = Player('lib/SPData/themes/braintrainer/btp_800x600.vlt')
    print pl.start('lib/CPData/Test_actData/ayb.avi')
    
    
    
    
    




# -*- coding: utf-8 -*-

# Copyright (c) 2008-2010 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
#
#           soundmemory.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("childsplay.soundmemory.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("childsplay.soundmemory")

# standard modules you probably need
import os,sys,random,glob

import pygame
from pygame.constants import *

import utils
from SPConstants import *
import SPSpriteUtils
import SPHelpText

# containers that can be used globally to store stuff
class Img:
    pass
class Snd:
    pass
class Global:
    """ Container to store all kind of stuff"""
    pass
    

class SndBut(SPSpriteUtils.SPSprite):
    Selected = None # This is a global reference to hold a selected object
    def __init__(self,snd,pos,img,img_high, name, scoreobs):
        self.snd = snd
        self.img = img
        self.img_high = img_high
        self.image = self.img
        SPSpriteUtils.SPSprite.__init__(self,self.image, name=name)
        self.rect = self.image.get_rect().move(pos)
        # self.image and self.rect are mandatory for the pygame sprite class
        self.high = 0 # used to signal highlight or not
        self.connect_callback(self.on_select_button, MOUSEBUTTONDOWN) # MOUSEBUTTONDOWN = pygame constant 
        self.scoreobs = scoreobs
    
    def set_number(self, s):
        self.img.blit(s, (0,0))
        self.img_high.blit(s, (0,0))
        self.image = self.img
        
    def play(self):
        self.snd.play()
    def stop(self):
        self.snd.stop()
        
    def highlight(self):
        if self.high:# we use a flag, because the images used are references.
            self.image = self.img
            self.high = 0
        else:
            self.image = self.img_high
            self.high = 1
        r = Img.screen.blit(self.image,self.rect)
        pygame.display.update(r)
        
    def on_update(self,*args):
        """ This is called when there's no callback function registered with
         the connect_callback method."""
        pass # we use callbacks, this is just as an example
    
    def on_select_button(self,obj,event,*args):
        """ This is a callback function used by a CPSprite object, like the callback
        fuctions in gtk++.
        A callback function must be connected to the object with a call to the 
        connect_callback method.
        When this fuction is called by the class it is called with a tuple of arguments.
        This tuple consist of:
            0 - a reference to the connected object.
            1 - the event that triggers this call.
            2 .. - data which where passed to the connect call.
        """        
        if not SndBut.Selected: # nothing selected yet
            self.play() # play the sound
            self.highlight() # toggle the high light button
            # keep track on how many times this card is opened
            # this is also stored in the dbase
            Global.selected_cards[self] += 1
            SndBut.Selected = self # store reference to compare when there's a second selection
            return
        if self.rect is SndBut.Selected.rect:# selected the same object twice, do nothing
            return
        # we are the second card
        self.play()
        self.highlight()
        Global.selected_cards[self] += 1
        pygame.time.wait(1500)
        if SndBut.Selected.name == self.name: # sounds are the same
            SndBut.Selected.kill()# remove sprite object
            SndBut.Selected.erase_sprite()
            self.kill()# remove 
            self.erase_sprite()
            SndBut.Selected = None   
            self.scoreobs(10)
        else:
            SndBut.Selected.highlight() #toggle highlight to normal
            self.highlight()
            SndBut.Selected = None
            self.scoreobs(-10)
            
class Activity:
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling loop.
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    helptitle (self) must return the title of the game can be localized.
    help (self) must return a list of strings describing the activty.
    name (self) must provide the activty name in english, not localized in lowercase.
  """

    def __init__(self,SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger =  logging.getLogger("childsplay.soundmemory.Activity")
        self.logger.info("Activity started")
        self.SPG = SPGoodies
        self.theme = self.SPG.get_theme()
        self.scoredisplay = self.SPG.get_scoredisplay()
        self.screen = self.SPG.get_screen()
        self.backgr = self.SPG.get_background()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        # store two surfaces for later use
        Img.screen = self.screen
        Img.backgr = self.backgr
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.libdir = self.SPG.get_libdir_path()
        self.my_datadir = os.path.join(self.libdir,'CPData','SoundmemoryData')
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'soundmemory.rc'))
        self.rchash['theme'] = self.theme
        self.logger.debug("found rc: %s" % self.rchash)
        # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
        # it returns a reference to the special CPGroup
        self.actives = SPSpriteUtils.SPInit(self.screen,self.backgr)
        self.beamer_set = 'off'
        
    def get_moviepath(self):
        movie = os.path.join(self.my_datadir,'help.avi')
        return movie
    
    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        self.actives.redraw()

    def clear_screen(self):
        self.screen.blit(self.orgscreen,self.blit_pos)
        self.backgr.blit(self.orgscreen,self.blit_pos)
        pygame.display.update()    
        
    def get_helptitle(self):
        """Mandatory method"""
        return _("Soundmemory")
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "soundmemory"
    
    def get_help(self):
        """Mandatory methods"""
        text = [_("The aim of this activity:"),
        _("Classic memory game where you have to find pairs of sounds."),
        " ", 
        _("When you touch the picture you will hear a sound.\nMatch the image with others that make the same sound,to get points."), 
        " "]
        return text 
    
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return _("Correctness is more important than speed")
        
    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Memory")
        
    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels" % number-of-levels)"""
        return _("This activity has %s levels") % 6
    
    def start(self):
        """Mandatory method."""
        if not pygame.mixer.get_init():
            self.logger.error("No sound card found or not available")
            self.SPG.tellcore_info_dialog(_(SPHelpText.Nosound.ActivityStart))
            raise utils.MyError(SPHelpText.Nosound.ActivityStart)
        self.all_sounds_files = glob.glob(os.path.join(self.my_datadir,'Sounds','*.ogg'))
        self.logger.debug("Loaded %s files from %s" % (len(self.all_sounds_files), \
                                                    os.path.join(self.my_datadir,'Sounds','*.ogg')))
        # We use the same images for all the buttons.
        # We check the sound objects for equality not the images.
        self.snd_img = utils.load_image(os.path.join(self.my_datadir,'but_bleu_up.png'))
        self.snd_img_high = utils.load_image(os.path.join(self.my_datadir,'but_red_down.png'))
        if self.beamer_set == 'on':
            self.snd_img_b = utils.load_image(os.path.join(self.my_datadir,'but_bleu_up_beamer.png'))
            self.snd_img_high_b = utils.load_image(os.path.join(self.my_datadir,'but_red_down_beamer.png'))
        # number of items y, number of items x, x offset, y offset
        self.gamelevels = [(2,3,200,100),(2,4,180,100),(3,4,180,60),(4,5,100,20),(4,6,50,20), (4, 7, 2,10 )]
        #self.SPG.tellcore_set_dice_minimal_level(1)
        self.displaydscore = 0
        self.levelrestartcounter = 0
        self.level = 1
        self.AreWeDT = False
    
    def dailytraining_pre_level(self, level):
        """Mandatory method"""
        pass

    def dailytraining_next_level(self, level, dbmapper):
        """Mandatory method.
        This should handle the DT logic"""
        self.AreWeDT = True
        self.SPG.tellcore_disable_level_indicator()
        self.next_level(level, dbmapper)
        return True    
    
    def pre_level(self, level):
        """Mandatory method
        Return True if you want the core to enter the eventloop after this call."""
        pass
        
    def next_level(self,level,dbmapper):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        if level > 6:
            return False
        if level == self.level:
            self.levelrestartcounter += 1
        self.level = level
        if self.blit_pos[1] == 0:
            ty = 10
        else:
            ty = 110
        
        # Lower framerate as we only check for mouse events and don't have animated
        # sprites
        self.SPG.tellcore_set_framerate(10)
        SndBut.Selected = None
        # make sure we don't have old sprites in the group
        self.actives.empty()
        # restore screen
        self.clear_screen()
        #pygame.display.update()
        
        r, c ,xoffset, yoffset = self.gamelevels[level-1]
        x_offset =  self.snd_img.get_width()+ 16
        y_offset = self.snd_img.get_height() + 16
        # shuffle sounds
        random.shuffle(self.all_sounds_files)
        num = (r * c)/2
        files = self.all_sounds_files[:num] * 2
        self.num_of_buttons = len(files)
        self.levelupcount = 1
        # db_mapper is a Python class object that provides easy access to the 
        # dbase.
        self.db_mapper = dbmapper
        # store number of cards into the db table 'cards' col
        self.db_mapper.insert('sounds',self.num_of_buttons)
        random.shuffle(files)
        # used to record how many times the same card is shown.
        # It's setup in the next_level method and used by the Card objects.
        # we store it into a class namespace to make it globally available.
        Global.selected_cards = {}
        i = 1
        for y in range(r):
            for x in range(c): 
                sfile = files.pop()
                
                snd = utils.load_sound(sfile)
                
                if self.beamer_set == 'on':
                    obj = SndBut(snd,\
                            (xoffset + x*x_offset,yoffset + y*y_offset+ty),\
                            self.snd_img_b.copy(),self.snd_img_high_b.copy(), sfile, self.scoreobs)
                    f = os.path.join(self.my_datadir, "%s.png" % i)
                    s = utils.load_image(f)
                    obj.set_number(s)
                    i += 1
                else:
                    obj = SndBut(snd,\
                            (xoffset + x*x_offset,yoffset + y*y_offset+ty),\
                            self.snd_img.copy(),self.snd_img_high.copy(), sfile, self.scoreobs)
                self.actives.add(obj) 
                obj.display_sprite()
                Global.selected_cards[obj] = 0
        return True
    
    def stop_timer(self):
        for obj in Global.selected_cards:
            obj.stop()
    
    def post_next_level(self):
        """Mandatory method.
        This is called by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a seperate thread like sound play."""
        pass 
    def scoreobs(self, i):
        """Called by the card object with a value to display as the score."""
        self.displaydscore += i
        if self.displaydscore <= 0:
            self.displaydscore = 0
        self.scoredisplay.set_score(self.displaydscore*5)
        
    def get_score(self,timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        # Same as the memory activities
        # T = Total cards
        # K = How many times the cards are selected. (a perfect game K == T)
        # F = Fraction of total result related to points. (maximum result is 10)
        # F1 = Fraction of result related to time
        # C = multiplier
        # S = time spend in seconds
        try:
            T = float(self.num_of_buttons * 2)
            K = float(self.knownbuttons)
        except AttributeError:
            return None
        m,s = timespend.split(':')
        S = float(int(m)*60 + int(s))
        F = 9.0
        F1 = 1.0
        C = 0.6
        C1 = 0.2
        points = F / max(K / T, 1.0 ) ** C1
        time = F1 / max(S / T , 1.0) ** C
        self.logger.info("@scoredata@ %s level %s T %s K %s S %s points %s time %s" %\
                          (self.get_name(), self.level, T, K, S, points, time))
        score = points + time
        return score
        
    def loop(self,events):
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""
        item = None
        for event in events:
            if event.type is MOUSEBUTTONDOWN:
                item = event
                break
        try:
            self.actives.update(item)# Check all objects in group for callback function
        except:
            self.logger.exception("Unhandled exception")
            raise StandardError
        # check if there objects left in the sprite group
        if not self.actives.sprites():
            # we call the SPGoodies observer to notify the core the level
            # is ended and we want to store the collected data
            num = 0
            for n in Global.selected_cards.values():
                num += n
            # store into dbase
            self.db_mapper.insert('knownsounds',num)
            # set an attribute so that 'get_score' can calculate the score
            self.knownbuttons = num
            if self.levelrestartcounter >= int(self.rchash[self.theme]['autolevel_value']):
                levelup = 1
                self.levelrestartcounter = 0
            else:
                levelup = 0
            if self.AreWeDT:
                        self.SPG.tellcore_level_end(level=self.level)
            else:
                self.SPG.tellcore_level_end(store_db=True, \
                                    level=min(6, self.level), \
                                    levelup=levelup)
        return 
        




# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
#
#           numbers_sp.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation. A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("childsplay.numbers_sp.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("childsplay.numbers_sp")

# standard modules you probably need
import os,sys
import random
import pygame
from pygame.constants import *

import utils
from SPConstants import *
import SPSpriteUtils
import SPWidgets

class Block(SPSpriteUtils.SPSprite):
    def __init__(self, ID, pos, white_img, white, green_img, red):
        self.ID = ID
        self.white_img = white_img
        self.white = white
        self.green_img = green_img
        self.red = red
        self.current_state = 'show'
        SPSpriteUtils.SPSprite.__init__(self, self.white)
        self.moveto(pos)
        
    def state_show(self):
        self.erase_sprite()
        self.image = self.white_img
        self.current_state = 'show'
        self.display_sprite()
    def state_closed(self):
        self.erase_sprite()
        self.image = self.white
        self.current_state = 'closed'
        self.display_sprite()
    def state_wrong(self):
        self.erase_sprite()
        self.image = self.red
        self.current_state = 'wrong'
        self.display_sprite()
    def state_good(self):
        self.erase_sprite()
        self.image = self.green_img
        self.current_state = 'good'
        self.display_sprite()
        
    def get_id(self):
        return self.ID

class BlockMaster:
    def __init__(self, datadir, actives, goodsnd, wrongsnd, observer):
        self.actives = actives
        self.datadir = datadir
        self.good = goodsnd
        self.wrong = wrongsnd
        self.observer = observer
        self.red = utils.load_image(os.path.join(datadir, 'r.png'))
        self.white = utils.load_image(os.path.join(datadir, 'w.png'))
        self.errors = 0
        self.totalblocks = 0
        
    def init(self, numbers, positions):
        self.totalblocks += len(numbers)
        self.blocklist = []
        for i in numbers:
            wimg = utils.load_image(os.path.join(self.datadir, 'w%s.png' % i))
            gimg = utils.load_image(os.path.join(self.datadir, 'g%s.png' % i))
            b = Block(i, positions[i-1], wimg, self.white, gimg, self.red)
            self.blocklist.append(b)
            b.connect_callback(self._cbf_block, MOUSEBUTTONDOWN, i)
        self.currentID = 1
    
    def show_all(self):
        self.actives.remove(self.blocklist)
        for b in self.blocklist:
            b.state_show()
            
    def close_all(self):
        for b in self.blocklist:
            b.state_closed()
        self.actives.add(self.blocklist)

    def _close_wrongs(self):
        for b in self.blocklist:
            if b.current_state == 'wrong':
                b.state_closed()

    def finished(self):
        return self.blocklist == []

    def get_errors(self):
        return self.errors

    def get_num_blocks(self):
        return self.totalblocks

    def _cbf_block(self, widget, event, data):
        ID = data[0]
        if self.currentID != ID:
            self.errors += 1
            self.wrong.play()
            widget.state_wrong()
            self.observer(-1)
        else:
            self.good.play()
            widget.state_good()
            self.blocklist.remove(widget)
            self.actives.remove(widget)
            self._close_wrongs()
            self.currentID = ID + 1
            self.observer(1)

class Activity:
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activty name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
  """

    def __init__(self,SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger =  logging.getLogger("childsplay.numbers_sp.Activity")
        self.logger.info("Activity started")
        self.SPG = SPGoodies
        self.lang = self.SPG.get_localesetting()[0][:2]
        self.scoredisplay = self.SPG.get_scoredisplay()
        self.theme = self.SPG.get_theme()
        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        # The location of the activities Data dir
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','Numbers_spData')
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
        # Location of the alphabet sounds dir
        self.absdir = self.SPG.get_absdir_path()# alphabet sounds dir
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'numbers_sp.rc'))
        self.logger.debug("rchash: " + str(self.rchash))
        # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
        # it returns a reference to the special CPGroup
        self.actives = SPSpriteUtils.SPInit(self.screen,self.backgr)
        
        self.GoodSound=utils.load_sound(os.path.join(self.CPdatadir, 'good.ogg'))
        self.WrongSound=utils.load_sound(os.path.join(self.CPdatadir, 'wrong.ogg'))
        self.Done=utils.load_sound(os.path.join(self.CPdatadir, 'wahoo.wav'))
        
        b = os.path.join(self.CPdatadir, '200px_80px_blue.png')
        b_ro = os.path.join(self.CPdatadir, '200px_80px_black.png')
        self.beginbut = SPWidgets.TransImgButton(b, b_ro, \
                                    (300, 510), fsize=24, text= _("Begin"),\
                                    fcol=WHITE)
        self.beginbut.connect_callback(self._cbf_begin, MOUSEBUTTONDOWN)
        self.cheatbut = SPWidgets.TransImgButton(b, b_ro, \
                                    (300, 510), fsize=24, text= _("Cheat"),\
                                    fcol=WHITE)
        self.cheatbut.connect_callback(self._cbf_cheat, MOUSEBUTTONDOWN)
        self.playfield = utils.load_image(os.path.join(self.my_datadir, "playfield.png")).convert()
        self.playfield_position = (40, 120)
        self.list_pos = []
        r = self.playfield.get_rect()
        x, y = self.playfield_position
        for i in range(((r.w - 10) / 70)):
            for j in range(((r.h - 10) / 70)):
                self.list_pos.append(((i * 70) + x + 10, (j* 70) + y + 10))
                    
    def clear_screen(self):
        self.screen.blit(self.orgscreen,self.blit_pos)
        self.backgr.blit(self.orgscreen,self.blit_pos)
        pygame.display.update()

    def get_moviepath(self):
        movie = os.path.join(self.my_datadir,'help.avi')
        return movie

    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        self.actives.refresh()

    def get_helptitle(self):
        """Mandatory method"""
        return _("Numbers_sp")
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "numbers_sp"
    
    def get_help(self):
        """Mandatory methods"""
        text = [_("The aim of this activity:"),
        _("You will see various numbers scattered around the screen."),
        " ", 
        _("Look carefully and try to remember their positions on the screen."), 
        _("Then push the 'BEGIN' button. The numbers will disappear."),
        _("Try to find the numbers, in the correct order and from low to high, by touching them on the screen."), 
        " "]
        return text
            
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return _("Correctness is more important than speed.")
        
    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Miscellaneous")
    
    def start(self):
        """Mandatory method."""
        self.AreWeDT = False
        
    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels") % number-of-levels"""
        return _("This activity has %s levels") % 6
    
    def pre_level(self,level):
        """Mandatory method.
        Return True to call the eventloop after this method is called."""
        self.logger.debug("pre_level called with: %s" % level)
        pass
        
    def next_level(self,level,dbmapper):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        self.logger.debug("nextlevel called with: %s" % level)
        self.level = level
        # number of exercises in one level
        if self.AreWeDT:
            self.exercises = 1
        else:
            self.exercises = int(self.rchash[self.theme]['exercises'])
        self.WEcheat = 0
        self.levelrestartcounter = 0
        self.numblocks = 3 + self.level
        self.totalblocks = 0
        self.blocksclicked = 0
        self.BM = BlockMaster(self.my_datadir, self.actives, \
                              self.GoodSound, self.WrongSound, self._observer)
        self.next_exercise()
        return True
        
    def _cbf_begin(self, *args):
        self.beginbut.erase_sprite()
        self.actives.remove(self.beginbut)
        self.cheatbut.display_sprite()
        self.actives.add(self.cheatbut)
        self.BM.close_all()

    def _cbf_cheat(self, *args):
        self.WEcheat += 1
        self.cheatbut.erase_sprite()
        self.actives.remove(self.cheatbut)
        self.beginbut.display_sprite()
        self.actives.add(self.beginbut)
        self.BM.show_all()

    def next_exercise(self):
        if not self.exercises:
            return True
        self.exercises -= 1
        self.levelrestartcounter += 1
        self.cheatbut.erase_sprite()
        self.actives.empty()
        self.actives.add(self.beginbut)
        
        x, y = self.playfield_position
        playfield = utils.load_image(os.path.join(self.my_datadir, "playfield.png")).convert()
        self.screen.blit(playfield, (x, y))
        pygame.display.update()
        self.backgr.blit(playfield, (x, y))
        self.BM.init(range(1, 4 + self.level), random.sample(self.list_pos, self.numblocks))
        self.BM.show_all()
        self.totalblocks += self.numblocks
        self.beginbut.display_sprite()

    def _observer(self, score):
        if score < 0:
            score = score * 10
        else:
            score = score * 15
        self.scoredisplay.increase_score(score)

    def dailytraining_pre_level(self, level):
        """Mandatory method"""
        pass
        
    def dailytraining_next_level(self, level, dbmapper):
        """Mandatory method.
        This should handle the DT logic"""
        self.AreWeDT = True
        self.SPG.tellcore_disable_level_indicator()
        self.next_level(level, dbmapper)
        return True       
        
    def post_next_level(self):
        """Mandatory method.
        This is called once by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a seperate thread like sound play."""
        pass 
        
    def get_score(self,timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        # T = Total blocks
        # K = Blocks clicked. (a perfect game K == T)
        # F = Fraction of total result related to points. (maximum result is 10)
        # F1 = Fraction of result related to time
        # C = multiplier
        # S = time spend in seconds
        try:
            T = float(self.totalblocks)
            K = float(self.BM.get_errors() + T + self.WEcheat)
        except AttributeError:
            return None
        m,s = timespend.split(':')
        S = float(int(m)*60 + int(s))
        F = 7.0
        F1 = 3.0
        C = 0.6
        C1 = 2.1
        points = F / max( K / T, 1.0 ) ** C1
        time = F1 / max(S / T , 1.0) ** C
        self.logger.info("@scoredata@ %s level %s T %s K %s S %s points %s time %s" % \
                         (self.get_name(), self.level, T, K, S, points, time))
        score = points + time
        return score
        
    def stop_timer(self):
        """You *must* provide a method to stop timers if you use them.
        The SPC will call this just in case and catch the exception in case 
        there's no 'stop_timer' method""" 
        try:
            self.timer.stop()
        except:
            pass

    def loop(self,events):
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""
        for event in events:
            if event.type in (MOUSEBUTTONDOWN, MOUSEMOTION):
                self.actives.update(event)
        if self.BM.finished():
            self.Done.play()
            pygame.time.wait(2000)
            self.clear_screen()
            if self.next_exercise():
                if self.levelrestartcounter >= int(self.rchash[self.theme]['autolevel_value']):
                    levelup = 1
                if self.AreWeDT:
                    self.SPG.tellcore_level_end(level=self.level)
                else:
                    self.SPG.tellcore_level_end(store_db=True, \
                                        level=min(6, self.level), \
                                                    levelup=levelup)
        return 
        
